import { join, dirname, extname, basename } from 'path';

import nodeResolve from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import replace from '@rollup/plugin-replace';
import json from '@rollup/plugin-json';
import babel from '@rollup/plugin-babel';
import clear from 'rollup-plugin-clear';
import progress from 'rollup-plugin-progress';
import nodePolyfills from 'rollup-plugin-node-polyfills';
import typescript from '@rollup/plugin-typescript';
import { terser } from 'rollup-plugin-terser';

import pkg from './package.json';

const isProd = process.env.NODE_ENV === 'production';

const formats = {
  commonjs: {
    format: 'cjs',
    file: pkg.main,
    sourcemap: true,
  },
  esm: {
    format: 'esm',
    file: pkg.module,
    sourcemap: true,
  },
  umd: {
    format: 'umd',
    file: pkg.browser,
    name: 'YWRequest',
    sourcemap: true,
    exports: 'named',
    globals: {
      'antd': 'antd'
    },
  },
};

const config = {
  input: './src/index.ts',
  inlineDynamicImports: true,
  output: [formats.esm, formats.commonjs],
  external: ['antd'],
  plugins: [
    clear({
      targets: ['lib'],
    }),
    progress({
      clearLine: false,
    }),
    replace({
      'process.env.DEBUG': 'false',
    }),
    nodeResolve({
      mainFields: ["jsnext", "preferBuiltins", "browser"],
      browser: true
    }),
    commonjs({browser: true}),
    json(),
    nodePolyfills(),
    babel(),
    typescript(),
  ],
  watch: {
    include: 'src/**',
    exclude: 'node_modules/**',
  },
};

if (isProd) {
  const file = formats.umd.file;

  config.output.push({
    ...formats.umd,
    file,
    plugins: [terser()],
  });
}

export default config;
