'use strict';

var antd = require('antd');
var require$$0$1 = require('fs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var require$$0__default = /*#__PURE__*/_interopDefaultLegacy(require$$0$1);

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

var axios$2 = {exports: {}};

var bind$2 = function bind(fn, thisArg) {
  return function wrap() {
    var args = new Array(arguments.length);

    for (var i = 0; i < args.length; i++) {
      args[i] = arguments[i];
    }

    return fn.apply(thisArg, args);
  };
};

var bind$1 = bind$2; // utils is a library of generic helper functions non-specific to axios

var toString = Object.prototype.toString;
/**
 * Determine if a value is an Array
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is an Array, otherwise false
 */

function isArray(val) {
  return toString.call(val) === '[object Array]';
}
/**
 * Determine if a value is undefined
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if the value is undefined, otherwise false
 */


function isUndefined(val) {
  return typeof val === 'undefined';
}
/**
 * Determine if a value is a Buffer
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Buffer, otherwise false
 */


function isBuffer(val) {
  return val !== null && !isUndefined(val) && val.constructor !== null && !isUndefined(val.constructor) && typeof val.constructor.isBuffer === 'function' && val.constructor.isBuffer(val);
}
/**
 * Determine if a value is an ArrayBuffer
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is an ArrayBuffer, otherwise false
 */


function isArrayBuffer(val) {
  return toString.call(val) === '[object ArrayBuffer]';
}
/**
 * Determine if a value is a FormData
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is an FormData, otherwise false
 */


function isFormData(val) {
  return typeof FormData !== 'undefined' && val instanceof FormData;
}
/**
 * Determine if a value is a view on an ArrayBuffer
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a view on an ArrayBuffer, otherwise false
 */


function isArrayBufferView(val) {
  var result;

  if (typeof ArrayBuffer !== 'undefined' && ArrayBuffer.isView) {
    result = ArrayBuffer.isView(val);
  } else {
    result = val && val.buffer && val.buffer instanceof ArrayBuffer;
  }

  return result;
}
/**
 * Determine if a value is a String
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a String, otherwise false
 */


function isString(val) {
  return typeof val === 'string';
}
/**
 * Determine if a value is a Number
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Number, otherwise false
 */


function isNumber(val) {
  return typeof val === 'number';
}
/**
 * Determine if a value is an Object
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is an Object, otherwise false
 */


function isObject(val) {
  return val !== null && typeof val === 'object';
}
/**
 * Determine if a value is a plain Object
 *
 * @param {Object} val The value to test
 * @return {boolean} True if value is a plain Object, otherwise false
 */


function isPlainObject(val) {
  if (toString.call(val) !== '[object Object]') {
    return false;
  }

  var prototype = Object.getPrototypeOf(val);
  return prototype === null || prototype === Object.prototype;
}
/**
 * Determine if a value is a Date
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Date, otherwise false
 */


function isDate(val) {
  return toString.call(val) === '[object Date]';
}
/**
 * Determine if a value is a File
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a File, otherwise false
 */


function isFile(val) {
  return toString.call(val) === '[object File]';
}
/**
 * Determine if a value is a Blob
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Blob, otherwise false
 */


function isBlob(val) {
  return toString.call(val) === '[object Blob]';
}
/**
 * Determine if a value is a Function
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Function, otherwise false
 */


function isFunction(val) {
  return toString.call(val) === '[object Function]';
}
/**
 * Determine if a value is a Stream
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a Stream, otherwise false
 */


function isStream(val) {
  return isObject(val) && isFunction(val.pipe);
}
/**
 * Determine if a value is a URLSearchParams object
 *
 * @param {Object} val The value to test
 * @returns {boolean} True if value is a URLSearchParams object, otherwise false
 */


function isURLSearchParams(val) {
  return typeof URLSearchParams !== 'undefined' && val instanceof URLSearchParams;
}
/**
 * Trim excess whitespace off the beginning and end of a string
 *
 * @param {String} str The String to trim
 * @returns {String} The String freed of excess whitespace
 */


function trim(str) {
  return str.trim ? str.trim() : str.replace(/^\s+|\s+$/g, '');
}
/**
 * Determine if we're running in a standard browser environment
 *
 * This allows axios to run in a web worker, and react-native.
 * Both environments support XMLHttpRequest, but not fully standard globals.
 *
 * web workers:
 *  typeof window -> undefined
 *  typeof document -> undefined
 *
 * react-native:
 *  navigator.product -> 'ReactNative'
 * nativescript
 *  navigator.product -> 'NativeScript' or 'NS'
 */


function isStandardBrowserEnv() {
  if (typeof navigator !== 'undefined' && (navigator.product === 'ReactNative' || navigator.product === 'NativeScript' || navigator.product === 'NS')) {
    return false;
  }

  return typeof window !== 'undefined' && typeof document !== 'undefined';
}
/**
 * Iterate over an Array or an Object invoking a function for each item.
 *
 * If `obj` is an Array callback will be called passing
 * the value, index, and complete array for each item.
 *
 * If 'obj' is an Object callback will be called passing
 * the value, key, and complete object for each property.
 *
 * @param {Object|Array} obj The object to iterate
 * @param {Function} fn The callback to invoke for each item
 */


function forEach(obj, fn) {
  // Don't bother if no value provided
  if (obj === null || typeof obj === 'undefined') {
    return;
  } // Force an array if not already something iterable


  if (typeof obj !== 'object') {
    /*eslint no-param-reassign:0*/
    obj = [obj];
  }

  if (isArray(obj)) {
    // Iterate over array values
    for (var i = 0, l = obj.length; i < l; i++) {
      fn.call(null, obj[i], i, obj);
    }
  } else {
    // Iterate over object keys
    for (var key in obj) {
      if (Object.prototype.hasOwnProperty.call(obj, key)) {
        fn.call(null, obj[key], key, obj);
      }
    }
  }
}
/**
 * Accepts varargs expecting each argument to be an object, then
 * immutably merges the properties of each object and returns result.
 *
 * When multiple objects contain the same key the later object in
 * the arguments list will take precedence.
 *
 * Example:
 *
 * ```js
 * var result = merge({foo: 123}, {foo: 456});
 * console.log(result.foo); // outputs 456
 * ```
 *
 * @param {Object} obj1 Object to merge
 * @returns {Object} Result of all merge properties
 */


function
  /* obj1, obj2, obj3, ... */
merge() {
  var result = {};

  function assignValue(val, key) {
    if (isPlainObject(result[key]) && isPlainObject(val)) {
      result[key] = merge(result[key], val);
    } else if (isPlainObject(val)) {
      result[key] = merge({}, val);
    } else if (isArray(val)) {
      result[key] = val.slice();
    } else {
      result[key] = val;
    }
  }

  for (var i = 0, l = arguments.length; i < l; i++) {
    forEach(arguments[i], assignValue);
  }

  return result;
}
/**
 * Extends object a by mutably adding to it the properties of object b.
 *
 * @param {Object} a The object to be extended
 * @param {Object} b The object to copy properties from
 * @param {Object} thisArg The object to bind function to
 * @return {Object} The resulting value of object a
 */


function extend(a, b, thisArg) {
  forEach(b, function assignValue(val, key) {
    if (thisArg && typeof val === 'function') {
      a[key] = bind$1(val, thisArg);
    } else {
      a[key] = val;
    }
  });
  return a;
}
/**
 * Remove byte order marker. This catches EF BB BF (the UTF-8 BOM)
 *
 * @param {string} content with BOM
 * @return {string} content value without BOM
 */


function stripBOM(content) {
  if (content.charCodeAt(0) === 0xFEFF) {
    content = content.slice(1);
  }

  return content;
}

var utils$d = {
  isArray: isArray,
  isArrayBuffer: isArrayBuffer,
  isBuffer: isBuffer,
  isFormData: isFormData,
  isArrayBufferView: isArrayBufferView,
  isString: isString,
  isNumber: isNumber,
  isObject: isObject,
  isPlainObject: isPlainObject,
  isUndefined: isUndefined,
  isDate: isDate,
  isFile: isFile,
  isBlob: isBlob,
  isFunction: isFunction,
  isStream: isStream,
  isURLSearchParams: isURLSearchParams,
  isStandardBrowserEnv: isStandardBrowserEnv,
  forEach: forEach,
  merge: merge,
  extend: extend,
  trim: trim,
  stripBOM: stripBOM
};

var utils$c = utils$d;

function encode(val) {
  return encodeURIComponent(val).replace(/%3A/gi, ':').replace(/%24/g, '$').replace(/%2C/gi, ',').replace(/%20/g, '+').replace(/%5B/gi, '[').replace(/%5D/gi, ']');
}
/**
 * Build a URL by appending params to the end
 *
 * @param {string} url The base of the url (e.g., http://www.google.com)
 * @param {object} [params] The params to be appended
 * @returns {string} The formatted url
 */


var buildURL$2 = function buildURL(url, params, paramsSerializer) {
  /*eslint no-param-reassign:0*/
  if (!params) {
    return url;
  }

  var serializedParams;

  if (paramsSerializer) {
    serializedParams = paramsSerializer(params);
  } else if (utils$c.isURLSearchParams(params)) {
    serializedParams = params.toString();
  } else {
    var parts = [];
    utils$c.forEach(params, function serialize(val, key) {
      if (val === null || typeof val === 'undefined') {
        return;
      }

      if (utils$c.isArray(val)) {
        key = key + '[]';
      } else {
        val = [val];
      }

      utils$c.forEach(val, function parseValue(v) {
        if (utils$c.isDate(v)) {
          v = v.toISOString();
        } else if (utils$c.isObject(v)) {
          v = JSON.stringify(v);
        }

        parts.push(encode(key) + '=' + encode(v));
      });
    });
    serializedParams = parts.join('&');
  }

  if (serializedParams) {
    var hashmarkIndex = url.indexOf('#');

    if (hashmarkIndex !== -1) {
      url = url.slice(0, hashmarkIndex);
    }

    url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
  }

  return url;
};

var utils$b = utils$d;

function InterceptorManager$1() {
  this.handlers = [];
}
/**
 * Add a new interceptor to the stack
 *
 * @param {Function} fulfilled The function to handle `then` for a `Promise`
 * @param {Function} rejected The function to handle `reject` for a `Promise`
 *
 * @return {Number} An ID used to remove interceptor later
 */


InterceptorManager$1.prototype.use = function use(fulfilled, rejected, options) {
  this.handlers.push({
    fulfilled: fulfilled,
    rejected: rejected,
    synchronous: options ? options.synchronous : false,
    runWhen: options ? options.runWhen : null
  });
  return this.handlers.length - 1;
};
/**
 * Remove an interceptor from the stack
 *
 * @param {Number} id The ID that was returned by `use`
 */


InterceptorManager$1.prototype.eject = function eject(id) {
  if (this.handlers[id]) {
    this.handlers[id] = null;
  }
};
/**
 * Iterate over all the registered interceptors
 *
 * This method is particularly useful for skipping over any
 * interceptors that may have become `null` calling `eject`.
 *
 * @param {Function} fn The function to call for each interceptor
 */


InterceptorManager$1.prototype.forEach = function forEach(fn) {
  utils$b.forEach(this.handlers, function forEachHandler(h) {
    if (h !== null) {
      fn(h);
    }
  });
};

var InterceptorManager_1 = InterceptorManager$1;

var utils$a = utils$d;

var normalizeHeaderName$1 = function normalizeHeaderName(headers, normalizedName) {
  utils$a.forEach(headers, function processHeader(value, name) {
    if (name !== normalizedName && name.toUpperCase() === normalizedName.toUpperCase()) {
      headers[normalizedName] = value;
      delete headers[name];
    }
  });
};

/**
 * Update an Error with the specified config, error code, and response.
 *
 * @param {Error} error The error to update.
 * @param {Object} config The config.
 * @param {string} [code] The error code (for example, 'ECONNABORTED').
 * @param {Object} [request] The request.
 * @param {Object} [response] The response.
 * @returns {Error} The error.
 */


var enhanceError$2 = function enhanceError(error, config, code, request, response) {
  error.config = config;

  if (code) {
    error.code = code;
  }

  error.request = request;
  error.response = response;
  error.isAxiosError = true;

  error.toJSON = function toJSON() {
    return {
      // Standard
      message: this.message,
      name: this.name,
      // Microsoft
      description: this.description,
      number: this.number,
      // Mozilla
      fileName: this.fileName,
      lineNumber: this.lineNumber,
      columnNumber: this.columnNumber,
      stack: this.stack,
      // Axios
      config: this.config,
      code: this.code
    };
  };

  return error;
};

var enhanceError$1 = enhanceError$2;
/**
 * Create an Error with the specified message, config, error code, request and response.
 *
 * @param {string} message The error message.
 * @param {Object} config The config.
 * @param {string} [code] The error code (for example, 'ECONNABORTED').
 * @param {Object} [request] The request.
 * @param {Object} [response] The response.
 * @returns {Error} The created error.
 */

var createError$2 = function createError(message, config, code, request, response) {
  var error = new Error(message);
  return enhanceError$1(error, config, code, request, response);
};

var createError$1 = createError$2;
/**
 * Resolve or reject a Promise based on response status.
 *
 * @param {Function} resolve A function that resolves the promise.
 * @param {Function} reject A function that rejects the promise.
 * @param {object} response The response.
 */

var settle$1 = function settle(resolve, reject, response) {
  var validateStatus = response.config.validateStatus;

  if (!response.status || !validateStatus || validateStatus(response.status)) {
    resolve(response);
  } else {
    reject(createError$1('Request failed with status code ' + response.status, response.config, null, response.request, response));
  }
};

var utils$9 = utils$d;
var cookies$1 = utils$9.isStandardBrowserEnv() ? // Standard browser envs support document.cookie
function standardBrowserEnv() {
  return {
    write: function write(name, value, expires, path, domain, secure) {
      var cookie = [];
      cookie.push(name + '=' + encodeURIComponent(value));

      if (utils$9.isNumber(expires)) {
        cookie.push('expires=' + new Date(expires).toGMTString());
      }

      if (utils$9.isString(path)) {
        cookie.push('path=' + path);
      }

      if (utils$9.isString(domain)) {
        cookie.push('domain=' + domain);
      }

      if (secure === true) {
        cookie.push('secure');
      }

      document.cookie = cookie.join('; ');
    },
    read: function read(name) {
      var match = document.cookie.match(new RegExp('(^|;\\s*)(' + name + ')=([^;]*)'));
      return match ? decodeURIComponent(match[3]) : null;
    },
    remove: function remove(name) {
      this.write(name, '', Date.now() - 86400000);
    }
  };
}() : // Non standard browser env (web workers, react-native) lack needed support.
function nonStandardBrowserEnv() {
  return {
    write: function write() {},
    read: function read() {
      return null;
    },
    remove: function remove() {}
  };
}();

/**
 * Determines whether the specified URL is absolute
 *
 * @param {string} url The URL to test
 * @returns {boolean} True if the specified URL is absolute, otherwise false
 */


var isAbsoluteURL$1 = function isAbsoluteURL(url) {
  // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
  // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
  // by any combination of letters, digits, plus, period, or hyphen.
  return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
};

/**
 * Creates a new URL by combining the specified URLs
 *
 * @param {string} baseURL The base URL
 * @param {string} relativeURL The relative URL
 * @returns {string} The combined URL
 */


var combineURLs$1 = function combineURLs(baseURL, relativeURL) {
  return relativeURL ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '') : baseURL;
};

var isAbsoluteURL = isAbsoluteURL$1;
var combineURLs = combineURLs$1;
/**
 * Creates a new URL by combining the baseURL with the requestedURL,
 * only when the requestedURL is not already an absolute URL.
 * If the requestURL is absolute, this function returns the requestedURL untouched.
 *
 * @param {string} baseURL The base URL
 * @param {string} requestedURL Absolute or relative URL to combine
 * @returns {string} The combined full path
 */

var buildFullPath$1 = function buildFullPath(baseURL, requestedURL) {
  if (baseURL && !isAbsoluteURL(requestedURL)) {
    return combineURLs(baseURL, requestedURL);
  }

  return requestedURL;
};

var utils$8 = utils$d; // Headers whose duplicates are ignored by node
// c.f. https://nodejs.org/api/http.html#http_message_headers

var ignoreDuplicateOf = ['age', 'authorization', 'content-length', 'content-type', 'etag', 'expires', 'from', 'host', 'if-modified-since', 'if-unmodified-since', 'last-modified', 'location', 'max-forwards', 'proxy-authorization', 'referer', 'retry-after', 'user-agent'];
/**
 * Parse headers into an object
 *
 * ```
 * Date: Wed, 27 Aug 2014 08:58:49 GMT
 * Content-Type: application/json
 * Connection: keep-alive
 * Transfer-Encoding: chunked
 * ```
 *
 * @param {String} headers Headers needing to be parsed
 * @returns {Object} Headers parsed into an object
 */

var parseHeaders$1 = function parseHeaders(headers) {
  var parsed = {};
  var key;
  var val;
  var i;

  if (!headers) {
    return parsed;
  }

  utils$8.forEach(headers.split('\n'), function parser(line) {
    i = line.indexOf(':');
    key = utils$8.trim(line.substr(0, i)).toLowerCase();
    val = utils$8.trim(line.substr(i + 1));

    if (key) {
      if (parsed[key] && ignoreDuplicateOf.indexOf(key) >= 0) {
        return;
      }

      if (key === 'set-cookie') {
        parsed[key] = (parsed[key] ? parsed[key] : []).concat([val]);
      } else {
        parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
      }
    }
  });
  return parsed;
};

var utils$7 = utils$d;
var isURLSameOrigin$1 = utils$7.isStandardBrowserEnv() ? // Standard browser envs have full support of the APIs needed to test
// whether the request URL is of the same origin as current location.
function standardBrowserEnv() {
  var msie = /(msie|trident)/i.test(navigator.userAgent);
  var urlParsingNode = document.createElement('a');
  var originURL;
  /**
  * Parse a URL to discover it's components
  *
  * @param {String} url The URL to be parsed
  * @returns {Object}
  */

  function resolveURL(url) {
    var href = url;

    if (msie) {
      // IE needs attribute set twice to normalize properties
      urlParsingNode.setAttribute('href', href);
      href = urlParsingNode.href;
    }

    urlParsingNode.setAttribute('href', href); // urlParsingNode provides the UrlUtils interface - http://url.spec.whatwg.org/#urlutils

    return {
      href: urlParsingNode.href,
      protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
      host: urlParsingNode.host,
      search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
      hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
      hostname: urlParsingNode.hostname,
      port: urlParsingNode.port,
      pathname: urlParsingNode.pathname.charAt(0) === '/' ? urlParsingNode.pathname : '/' + urlParsingNode.pathname
    };
  }

  originURL = resolveURL(window.location.href);
  /**
  * Determine if a URL shares the same origin as the current location
  *
  * @param {String} requestURL The URL to test
  * @returns {boolean} True if URL shares the same origin, otherwise false
  */

  return function isURLSameOrigin(requestURL) {
    var parsed = utils$7.isString(requestURL) ? resolveURL(requestURL) : requestURL;
    return parsed.protocol === originURL.protocol && parsed.host === originURL.host;
  };
}() : // Non standard browser envs (web workers, react-native) lack needed support.
function nonStandardBrowserEnv() {
  return function isURLSameOrigin() {
    return true;
  };
}();

var utils$6 = utils$d;
var settle = settle$1;
var cookies = cookies$1;
var buildURL$1 = buildURL$2;
var buildFullPath = buildFullPath$1;
var parseHeaders = parseHeaders$1;
var isURLSameOrigin = isURLSameOrigin$1;
var createError = createError$2;

var xhr = function xhrAdapter(config) {
  return new Promise(function dispatchXhrRequest(resolve, reject) {
    var requestData = config.data;
    var requestHeaders = config.headers;
    var responseType = config.responseType;

    if (utils$6.isFormData(requestData)) {
      delete requestHeaders['Content-Type']; // Let the browser set it
    }

    var request = new XMLHttpRequest(); // HTTP basic authentication

    if (config.auth) {
      var username = config.auth.username || '';
      var password = config.auth.password ? unescape(encodeURIComponent(config.auth.password)) : '';
      requestHeaders.Authorization = 'Basic ' + btoa(username + ':' + password);
    }

    var fullPath = buildFullPath(config.baseURL, config.url);
    request.open(config.method.toUpperCase(), buildURL$1(fullPath, config.params, config.paramsSerializer), true); // Set the request timeout in MS

    request.timeout = config.timeout;

    function onloadend() {
      if (!request) {
        return;
      } // Prepare the response


      var responseHeaders = 'getAllResponseHeaders' in request ? parseHeaders(request.getAllResponseHeaders()) : null;
      var responseData = !responseType || responseType === 'text' || responseType === 'json' ? request.responseText : request.response;
      var response = {
        data: responseData,
        status: request.status,
        statusText: request.statusText,
        headers: responseHeaders,
        config: config,
        request: request
      };
      settle(resolve, reject, response); // Clean up request

      request = null;
    }

    if ('onloadend' in request) {
      // Use onloadend if available
      request.onloadend = onloadend;
    } else {
      // Listen for ready state to emulate onloadend
      request.onreadystatechange = function handleLoad() {
        if (!request || request.readyState !== 4) {
          return;
        } // The request errored out and we didn't get a response, this will be
        // handled by onerror instead
        // With one exception: request that using file: protocol, most browsers
        // will return status as 0 even though it's a successful request


        if (request.status === 0 && !(request.responseURL && request.responseURL.indexOf('file:') === 0)) {
          return;
        } // readystate handler is calling before onerror or ontimeout handlers,
        // so we should call onloadend on the next 'tick'


        setTimeout(onloadend);
      };
    } // Handle browser request cancellation (as opposed to a manual cancellation)


    request.onabort = function handleAbort() {
      if (!request) {
        return;
      }

      reject(createError('Request aborted', config, 'ECONNABORTED', request)); // Clean up request

      request = null;
    }; // Handle low level network errors


    request.onerror = function handleError() {
      // Real errors are hidden from us by the browser
      // onerror should only fire if it's a network error
      reject(createError('Network Error', config, null, request)); // Clean up request

      request = null;
    }; // Handle timeout


    request.ontimeout = function handleTimeout() {
      var timeoutErrorMessage = 'timeout of ' + config.timeout + 'ms exceeded';

      if (config.timeoutErrorMessage) {
        timeoutErrorMessage = config.timeoutErrorMessage;
      }

      reject(createError(timeoutErrorMessage, config, config.transitional && config.transitional.clarifyTimeoutError ? 'ETIMEDOUT' : 'ECONNABORTED', request)); // Clean up request

      request = null;
    }; // Add xsrf header
    // This is only done if running in a standard browser environment.
    // Specifically not if we're in a web worker, or react-native.


    if (utils$6.isStandardBrowserEnv()) {
      // Add xsrf header
      var xsrfValue = (config.withCredentials || isURLSameOrigin(fullPath)) && config.xsrfCookieName ? cookies.read(config.xsrfCookieName) : undefined;

      if (xsrfValue) {
        requestHeaders[config.xsrfHeaderName] = xsrfValue;
      }
    } // Add headers to the request


    if ('setRequestHeader' in request) {
      utils$6.forEach(requestHeaders, function setRequestHeader(val, key) {
        if (typeof requestData === 'undefined' && key.toLowerCase() === 'content-type') {
          // Remove Content-Type if data is undefined
          delete requestHeaders[key];
        } else {
          // Otherwise add header to the request
          request.setRequestHeader(key, val);
        }
      });
    } // Add withCredentials to request if needed


    if (!utils$6.isUndefined(config.withCredentials)) {
      request.withCredentials = !!config.withCredentials;
    } // Add responseType to request if needed


    if (responseType && responseType !== 'json') {
      request.responseType = config.responseType;
    } // Handle progress if needed


    if (typeof config.onDownloadProgress === 'function') {
      request.addEventListener('progress', config.onDownloadProgress);
    } // Not all browsers support upload events


    if (typeof config.onUploadProgress === 'function' && request.upload) {
      request.upload.addEventListener('progress', config.onUploadProgress);
    }

    if (config.cancelToken) {
      // Handle cancellation
      config.cancelToken.promise.then(function onCanceled(cancel) {
        if (!request) {
          return;
        }

        request.abort();
        reject(cancel); // Clean up request

        request = null;
      });
    }

    if (!requestData) {
      requestData = null;
    } // Send the request


    request.send(requestData);
  });
};

var utils$5 = utils$d;
var normalizeHeaderName = normalizeHeaderName$1;
var enhanceError = enhanceError$2;
var DEFAULT_CONTENT_TYPE = {
  'Content-Type': 'application/x-www-form-urlencoded'
};

function setContentTypeIfUnset(headers, value) {
  if (!utils$5.isUndefined(headers) && utils$5.isUndefined(headers['Content-Type'])) {
    headers['Content-Type'] = value;
  }
}

function getDefaultAdapter() {
  var adapter;

  if (typeof XMLHttpRequest !== 'undefined') {
    // For browsers use XHR adapter
    adapter = xhr;
  } else if (typeof process !== 'undefined' && Object.prototype.toString.call(process) === '[object process]') {
    // For node use HTTP adapter
    adapter = xhr;
  }

  return adapter;
}

function stringifySafely(rawValue, parser, encoder) {
  if (utils$5.isString(rawValue)) {
    try {
      (parser || JSON.parse)(rawValue);
      return utils$5.trim(rawValue);
    } catch (e) {
      if (e.name !== 'SyntaxError') {
        throw e;
      }
    }
  }

  return (encoder || JSON.stringify)(rawValue);
}

var defaults$3 = {
  transitional: {
    silentJSONParsing: true,
    forcedJSONParsing: true,
    clarifyTimeoutError: false
  },
  adapter: getDefaultAdapter(),
  transformRequest: [function transformRequest(data, headers) {
    normalizeHeaderName(headers, 'Accept');
    normalizeHeaderName(headers, 'Content-Type');

    if (utils$5.isFormData(data) || utils$5.isArrayBuffer(data) || utils$5.isBuffer(data) || utils$5.isStream(data) || utils$5.isFile(data) || utils$5.isBlob(data)) {
      return data;
    }

    if (utils$5.isArrayBufferView(data)) {
      return data.buffer;
    }

    if (utils$5.isURLSearchParams(data)) {
      setContentTypeIfUnset(headers, 'application/x-www-form-urlencoded;charset=utf-8');
      return data.toString();
    }

    if (utils$5.isObject(data) || headers && headers['Content-Type'] === 'application/json') {
      setContentTypeIfUnset(headers, 'application/json');
      return stringifySafely(data);
    }

    return data;
  }],
  transformResponse: [function transformResponse(data) {
    var transitional = this.transitional;
    var silentJSONParsing = transitional && transitional.silentJSONParsing;
    var forcedJSONParsing = transitional && transitional.forcedJSONParsing;
    var strictJSONParsing = !silentJSONParsing && this.responseType === 'json';

    if (strictJSONParsing || forcedJSONParsing && utils$5.isString(data) && data.length) {
      try {
        return JSON.parse(data);
      } catch (e) {
        if (strictJSONParsing) {
          if (e.name === 'SyntaxError') {
            throw enhanceError(e, this, 'E_JSON_PARSE');
          }

          throw e;
        }
      }
    }

    return data;
  }],

  /**
   * A timeout in milliseconds to abort a request. If set to 0 (default) a
   * timeout is not created.
   */
  timeout: 0,
  xsrfCookieName: 'XSRF-TOKEN',
  xsrfHeaderName: 'X-XSRF-TOKEN',
  maxContentLength: -1,
  maxBodyLength: -1,
  validateStatus: function validateStatus(status) {
    return status >= 200 && status < 300;
  }
};
defaults$3.headers = {
  common: {
    'Accept': 'application/json, text/plain, */*'
  }
};
utils$5.forEach(['delete', 'get', 'head'], function forEachMethodNoData(method) {
  defaults$3.headers[method] = {};
});
utils$5.forEach(['post', 'put', 'patch'], function forEachMethodWithData(method) {
  defaults$3.headers[method] = utils$5.merge(DEFAULT_CONTENT_TYPE);
});
var defaults_1 = defaults$3;

var utils$4 = utils$d;
var defaults$2 = defaults_1;
/**
 * Transform the data for a request or a response
 *
 * @param {Object|String} data The data to be transformed
 * @param {Array} headers The headers for the request or response
 * @param {Array|Function} fns A single function or Array of functions
 * @returns {*} The resulting transformed data
 */

var transformData$1 = function transformData(data, headers, fns) {
  var context = this || defaults$2;
  /*eslint no-param-reassign:0*/

  utils$4.forEach(fns, function transform(fn) {
    data = fn.call(context, data, headers);
  });
  return data;
};

var isCancel$1 = function isCancel(value) {
  return !!(value && value.__CANCEL__);
};

var utils$3 = utils$d;
var transformData = transformData$1;
var isCancel = isCancel$1;
var defaults$1 = defaults_1;
/**
 * Throws a `Cancel` if cancellation has been requested.
 */

function throwIfCancellationRequested(config) {
  if (config.cancelToken) {
    config.cancelToken.throwIfRequested();
  }
}
/**
 * Dispatch a request to the server using the configured adapter.
 *
 * @param {object} config The config that is to be used for the request
 * @returns {Promise} The Promise to be fulfilled
 */


var dispatchRequest$1 = function dispatchRequest(config) {
  throwIfCancellationRequested(config); // Ensure headers exist

  config.headers = config.headers || {}; // Transform request data

  config.data = transformData.call(config, config.data, config.headers, config.transformRequest); // Flatten headers

  config.headers = utils$3.merge(config.headers.common || {}, config.headers[config.method] || {}, config.headers);
  utils$3.forEach(['delete', 'get', 'head', 'post', 'put', 'patch', 'common'], function cleanHeaderConfig(method) {
    delete config.headers[method];
  });
  var adapter = config.adapter || defaults$1.adapter;
  return adapter(config).then(function onAdapterResolution(response) {
    throwIfCancellationRequested(config); // Transform response data

    response.data = transformData.call(config, response.data, response.headers, config.transformResponse);
    return response;
  }, function onAdapterRejection(reason) {
    if (!isCancel(reason)) {
      throwIfCancellationRequested(config); // Transform response data

      if (reason && reason.response) {
        reason.response.data = transformData.call(config, reason.response.data, reason.response.headers, config.transformResponse);
      }
    }

    return Promise.reject(reason);
  });
};

var utils$2 = utils$d;
/**
 * Config-specific merge-function which creates a new config-object
 * by merging two configuration objects together.
 *
 * @param {Object} config1
 * @param {Object} config2
 * @returns {Object} New object resulting from merging config2 to config1
 */

var mergeConfig$2 = function mergeConfig(config1, config2) {
  // eslint-disable-next-line no-param-reassign
  config2 = config2 || {};
  var config = {};
  var valueFromConfig2Keys = ['url', 'method', 'data'];
  var mergeDeepPropertiesKeys = ['headers', 'auth', 'proxy', 'params'];
  var defaultToConfig2Keys = ['baseURL', 'transformRequest', 'transformResponse', 'paramsSerializer', 'timeout', 'timeoutMessage', 'withCredentials', 'adapter', 'responseType', 'xsrfCookieName', 'xsrfHeaderName', 'onUploadProgress', 'onDownloadProgress', 'decompress', 'maxContentLength', 'maxBodyLength', 'maxRedirects', 'transport', 'httpAgent', 'httpsAgent', 'cancelToken', 'socketPath', 'responseEncoding'];
  var directMergeKeys = ['validateStatus'];

  function getMergedValue(target, source) {
    if (utils$2.isPlainObject(target) && utils$2.isPlainObject(source)) {
      return utils$2.merge(target, source);
    } else if (utils$2.isPlainObject(source)) {
      return utils$2.merge({}, source);
    } else if (utils$2.isArray(source)) {
      return source.slice();
    }

    return source;
  }

  function mergeDeepProperties(prop) {
    if (!utils$2.isUndefined(config2[prop])) {
      config[prop] = getMergedValue(config1[prop], config2[prop]);
    } else if (!utils$2.isUndefined(config1[prop])) {
      config[prop] = getMergedValue(undefined, config1[prop]);
    }
  }

  utils$2.forEach(valueFromConfig2Keys, function valueFromConfig2(prop) {
    if (!utils$2.isUndefined(config2[prop])) {
      config[prop] = getMergedValue(undefined, config2[prop]);
    }
  });
  utils$2.forEach(mergeDeepPropertiesKeys, mergeDeepProperties);
  utils$2.forEach(defaultToConfig2Keys, function defaultToConfig2(prop) {
    if (!utils$2.isUndefined(config2[prop])) {
      config[prop] = getMergedValue(undefined, config2[prop]);
    } else if (!utils$2.isUndefined(config1[prop])) {
      config[prop] = getMergedValue(undefined, config1[prop]);
    }
  });
  utils$2.forEach(directMergeKeys, function merge(prop) {
    if (prop in config2) {
      config[prop] = getMergedValue(config1[prop], config2[prop]);
    } else if (prop in config1) {
      config[prop] = getMergedValue(undefined, config1[prop]);
    }
  });
  var axiosKeys = valueFromConfig2Keys.concat(mergeDeepPropertiesKeys).concat(defaultToConfig2Keys).concat(directMergeKeys);
  var otherKeys = Object.keys(config1).concat(Object.keys(config2)).filter(function filterAxiosKeys(key) {
    return axiosKeys.indexOf(key) === -1;
  });
  utils$2.forEach(otherKeys, mergeDeepProperties);
  return config;
};

var name = "axios";
var version = "0.21.4";
var description = "Promise based HTTP client for the browser and node.js";
var main = "index.js";
var scripts = {
	test: "grunt test",
	start: "node ./sandbox/server.js",
	build: "NODE_ENV=production grunt build",
	preversion: "npm test",
	version: "npm run build && grunt version && git add -A dist && git add CHANGELOG.md bower.json package.json",
	postversion: "git push && git push --tags",
	examples: "node ./examples/server.js",
	coveralls: "cat coverage/lcov.info | ./node_modules/coveralls/bin/coveralls.js",
	fix: "eslint --fix lib/**/*.js"
};
var repository = {
	type: "git",
	url: "https://github.com/axios/axios.git"
};
var keywords = [
	"xhr",
	"http",
	"ajax",
	"promise",
	"node"
];
var author = "Matt Zabriskie";
var license = "MIT";
var bugs = {
	url: "https://github.com/axios/axios/issues"
};
var homepage = "https://axios-http.com";
var devDependencies = {
	coveralls: "^3.0.0",
	"es6-promise": "^4.2.4",
	grunt: "^1.3.0",
	"grunt-banner": "^0.6.0",
	"grunt-cli": "^1.2.0",
	"grunt-contrib-clean": "^1.1.0",
	"grunt-contrib-watch": "^1.0.0",
	"grunt-eslint": "^23.0.0",
	"grunt-karma": "^4.0.0",
	"grunt-mocha-test": "^0.13.3",
	"grunt-ts": "^6.0.0-beta.19",
	"grunt-webpack": "^4.0.2",
	"istanbul-instrumenter-loader": "^1.0.0",
	"jasmine-core": "^2.4.1",
	karma: "^6.3.2",
	"karma-chrome-launcher": "^3.1.0",
	"karma-firefox-launcher": "^2.1.0",
	"karma-jasmine": "^1.1.1",
	"karma-jasmine-ajax": "^0.1.13",
	"karma-safari-launcher": "^1.0.0",
	"karma-sauce-launcher": "^4.3.6",
	"karma-sinon": "^1.0.5",
	"karma-sourcemap-loader": "^0.3.8",
	"karma-webpack": "^4.0.2",
	"load-grunt-tasks": "^3.5.2",
	minimist: "^1.2.0",
	mocha: "^8.2.1",
	sinon: "^4.5.0",
	"terser-webpack-plugin": "^4.2.3",
	typescript: "^4.0.5",
	"url-search-params": "^0.10.0",
	webpack: "^4.44.2",
	"webpack-dev-server": "^3.11.0"
};
var browser = {
	"./lib/adapters/http.js": "./lib/adapters/xhr.js"
};
var jsdelivr = "dist/axios.min.js";
var unpkg = "dist/axios.min.js";
var typings = "./index.d.ts";
var dependencies = {
	"follow-redirects": "^1.14.0"
};
var bundlesize = [
	{
		path: "./dist/axios.min.js",
		threshold: "5kB"
	}
];
var require$$0 = {
	name: name,
	version: version,
	description: description,
	main: main,
	scripts: scripts,
	repository: repository,
	keywords: keywords,
	author: author,
	license: license,
	bugs: bugs,
	homepage: homepage,
	devDependencies: devDependencies,
	browser: browser,
	jsdelivr: jsdelivr,
	unpkg: unpkg,
	typings: typings,
	dependencies: dependencies,
	bundlesize: bundlesize
};

var pkg = require$$0;
var validators$1 = {}; // eslint-disable-next-line func-names

['object', 'boolean', 'number', 'function', 'string', 'symbol'].forEach(function (type, i) {
  validators$1[type] = function validator(thing) {
    return typeof thing === type || 'a' + (i < 1 ? 'n ' : ' ') + type;
  };
});
var deprecatedWarnings = {};
var currentVerArr = pkg.version.split('.');
/**
 * Compare package versions
 * @param {string} version
 * @param {string?} thanVersion
 * @returns {boolean}
 */

function isOlderVersion(version, thanVersion) {
  var pkgVersionArr = thanVersion ? thanVersion.split('.') : currentVerArr;
  var destVer = version.split('.');

  for (var i = 0; i < 3; i++) {
    if (pkgVersionArr[i] > destVer[i]) {
      return true;
    } else if (pkgVersionArr[i] < destVer[i]) {
      return false;
    }
  }

  return false;
}
/**
 * Transitional option validator
 * @param {function|boolean?} validator
 * @param {string?} version
 * @param {string} message
 * @returns {function}
 */


validators$1.transitional = function transitional(validator, version, message) {
  var isDeprecated = version && isOlderVersion(version);

  function formatMessage(opt, desc) {
    return '[Axios v' + pkg.version + '] Transitional option \'' + opt + '\'' + desc + (message ? '. ' + message : '');
  } // eslint-disable-next-line func-names


  return function (value, opt, opts) {
    if (validator === false) {
      throw new Error(formatMessage(opt, ' has been removed in ' + version));
    }

    if (isDeprecated && !deprecatedWarnings[opt]) {
      deprecatedWarnings[opt] = true; // eslint-disable-next-line no-console

      console.warn(formatMessage(opt, ' has been deprecated since v' + version + ' and will be removed in the near future'));
    }

    return validator ? validator(value, opt, opts) : true;
  };
};
/**
 * Assert object's properties type
 * @param {object} options
 * @param {object} schema
 * @param {boolean?} allowUnknown
 */


function assertOptions(options, schema, allowUnknown) {
  if (typeof options !== 'object') {
    throw new TypeError('options must be an object');
  }

  var keys = Object.keys(options);
  var i = keys.length;

  while (i-- > 0) {
    var opt = keys[i];
    var validator = schema[opt];

    if (validator) {
      var value = options[opt];
      var result = value === undefined || validator(value, opt, options);

      if (result !== true) {
        throw new TypeError('option ' + opt + ' must be ' + result);
      }

      continue;
    }

    if (allowUnknown !== true) {
      throw Error('Unknown option ' + opt);
    }
  }
}

var validator$1 = {
  isOlderVersion: isOlderVersion,
  assertOptions: assertOptions,
  validators: validators$1
};

var utils$1 = utils$d;
var buildURL = buildURL$2;
var InterceptorManager = InterceptorManager_1;
var dispatchRequest = dispatchRequest$1;
var mergeConfig$1 = mergeConfig$2;
var validator = validator$1;
var validators = validator.validators;
/**
 * Create a new instance of Axios
 *
 * @param {Object} instanceConfig The default config for the instance
 */

function Axios$1(instanceConfig) {
  this.defaults = instanceConfig;
  this.interceptors = {
    request: new InterceptorManager(),
    response: new InterceptorManager()
  };
}
/**
 * Dispatch a request
 *
 * @param {Object} config The config specific for this request (merged with this.defaults)
 */


Axios$1.prototype.request = function request(config) {
  /*eslint no-param-reassign:0*/
  // Allow for axios('example/url'[, config]) a la fetch API
  if (typeof config === 'string') {
    config = arguments[1] || {};
    config.url = arguments[0];
  } else {
    config = config || {};
  }

  config = mergeConfig$1(this.defaults, config); // Set config.method

  if (config.method) {
    config.method = config.method.toLowerCase();
  } else if (this.defaults.method) {
    config.method = this.defaults.method.toLowerCase();
  } else {
    config.method = 'get';
  }

  var transitional = config.transitional;

  if (transitional !== undefined) {
    validator.assertOptions(transitional, {
      silentJSONParsing: validators.transitional(validators.boolean, '1.0.0'),
      forcedJSONParsing: validators.transitional(validators.boolean, '1.0.0'),
      clarifyTimeoutError: validators.transitional(validators.boolean, '1.0.0')
    }, false);
  } // filter out skipped interceptors


  var requestInterceptorChain = [];
  var synchronousRequestInterceptors = true;
  this.interceptors.request.forEach(function unshiftRequestInterceptors(interceptor) {
    if (typeof interceptor.runWhen === 'function' && interceptor.runWhen(config) === false) {
      return;
    }

    synchronousRequestInterceptors = synchronousRequestInterceptors && interceptor.synchronous;
    requestInterceptorChain.unshift(interceptor.fulfilled, interceptor.rejected);
  });
  var responseInterceptorChain = [];
  this.interceptors.response.forEach(function pushResponseInterceptors(interceptor) {
    responseInterceptorChain.push(interceptor.fulfilled, interceptor.rejected);
  });
  var promise;

  if (!synchronousRequestInterceptors) {
    var chain = [dispatchRequest, undefined];
    Array.prototype.unshift.apply(chain, requestInterceptorChain);
    chain = chain.concat(responseInterceptorChain);
    promise = Promise.resolve(config);

    while (chain.length) {
      promise = promise.then(chain.shift(), chain.shift());
    }

    return promise;
  }

  var newConfig = config;

  while (requestInterceptorChain.length) {
    var onFulfilled = requestInterceptorChain.shift();
    var onRejected = requestInterceptorChain.shift();

    try {
      newConfig = onFulfilled(newConfig);
    } catch (error) {
      onRejected(error);
      break;
    }
  }

  try {
    promise = dispatchRequest(newConfig);
  } catch (error) {
    return Promise.reject(error);
  }

  while (responseInterceptorChain.length) {
    promise = promise.then(responseInterceptorChain.shift(), responseInterceptorChain.shift());
  }

  return promise;
};

Axios$1.prototype.getUri = function getUri(config) {
  config = mergeConfig$1(this.defaults, config);
  return buildURL(config.url, config.params, config.paramsSerializer).replace(/^\?/, '');
}; // Provide aliases for supported request methods


utils$1.forEach(['delete', 'get', 'head', 'options'], function forEachMethodNoData(method) {
  /*eslint func-names:0*/
  Axios$1.prototype[method] = function (url, config) {
    return this.request(mergeConfig$1(config || {}, {
      method: method,
      url: url,
      data: (config || {}).data
    }));
  };
});
utils$1.forEach(['post', 'put', 'patch'], function forEachMethodWithData(method) {
  /*eslint func-names:0*/
  Axios$1.prototype[method] = function (url, data, config) {
    return this.request(mergeConfig$1(config || {}, {
      method: method,
      url: url,
      data: data
    }));
  };
});
var Axios_1 = Axios$1;

/**
 * A `Cancel` is an object that is thrown when an operation is canceled.
 *
 * @class
 * @param {string=} message The message.
 */


function Cancel$1(message) {
  this.message = message;
}

Cancel$1.prototype.toString = function toString() {
  return 'Cancel' + (this.message ? ': ' + this.message : '');
};

Cancel$1.prototype.__CANCEL__ = true;
var Cancel_1 = Cancel$1;

var Cancel = Cancel_1;
/**
 * A `CancelToken` is an object that can be used to request cancellation of an operation.
 *
 * @class
 * @param {Function} executor The executor function.
 */

function CancelToken(executor) {
  if (typeof executor !== 'function') {
    throw new TypeError('executor must be a function.');
  }

  var resolvePromise;
  this.promise = new Promise(function promiseExecutor(resolve) {
    resolvePromise = resolve;
  });
  var token = this;
  executor(function cancel(message) {
    if (token.reason) {
      // Cancellation has already been requested
      return;
    }

    token.reason = new Cancel(message);
    resolvePromise(token.reason);
  });
}
/**
 * Throws a `Cancel` if cancellation has been requested.
 */


CancelToken.prototype.throwIfRequested = function throwIfRequested() {
  if (this.reason) {
    throw this.reason;
  }
};
/**
 * Returns an object that contains a new `CancelToken` and a function that, when called,
 * cancels the `CancelToken`.
 */


CancelToken.source = function source() {
  var cancel;
  var token = new CancelToken(function executor(c) {
    cancel = c;
  });
  return {
    token: token,
    cancel: cancel
  };
};

var CancelToken_1 = CancelToken;

/**
 * Syntactic sugar for invoking a function and expanding an array for arguments.
 *
 * Common use case would be to use `Function.prototype.apply`.
 *
 *  ```js
 *  function f(x, y, z) {}
 *  var args = [1, 2, 3];
 *  f.apply(null, args);
 *  ```
 *
 * With `spread` this example can be re-written.
 *
 *  ```js
 *  spread(function(x, y, z) {})([1, 2, 3]);
 *  ```
 *
 * @param {Function} callback
 * @returns {Function}
 */


var spread = function spread(callback) {
  return function wrap(arr) {
    return callback.apply(null, arr);
  };
};

/**
 * Determines whether the payload is an error thrown by Axios
 *
 * @param {*} payload The value to test
 * @returns {boolean} True if the payload is an error thrown by Axios, otherwise false
 */


var isAxiosError = function isAxiosError(payload) {
  return typeof payload === 'object' && payload.isAxiosError === true;
};

var utils = utils$d;
var bind = bind$2;
var Axios = Axios_1;
var mergeConfig = mergeConfig$2;
var defaults = defaults_1;
/**
 * Create an instance of Axios
 *
 * @param {Object} defaultConfig The default config for the instance
 * @return {Axios} A new instance of Axios
 */

function createInstance(defaultConfig) {
  var context = new Axios(defaultConfig);
  var instance = bind(Axios.prototype.request, context); // Copy axios.prototype to instance

  utils.extend(instance, Axios.prototype, context); // Copy context to instance

  utils.extend(instance, context);
  return instance;
} // Create the default instance to be exported


var axios$1 = createInstance(defaults); // Expose Axios class to allow class inheritance

axios$1.Axios = Axios; // Factory for creating new instances

axios$1.create = function create(instanceConfig) {
  return createInstance(mergeConfig(axios$1.defaults, instanceConfig));
}; // Expose Cancel & CancelToken


axios$1.Cancel = Cancel_1;
axios$1.CancelToken = CancelToken_1;
axios$1.isCancel = isCancel$1; // Expose all/spread

axios$1.all = function all(promises) {
  return Promise.all(promises);
};

axios$1.spread = spread; // Expose isAxiosError

axios$1.isAxiosError = isAxiosError;
axios$2.exports = axios$1; // Allow use of default import syntax in TypeScript

axios$2.exports.default = axios$1;

var axios = axios$2.exports;

var ywfeUtils_umd = {exports: {}};

(function (module, exports) {
  !function (e, t) {
    t(exports, require$$0__default["default"]) ;
  }(commonjsGlobal, function (e, t) {

    function r(e) {
      return e && "object" == typeof e && "default" in e ? e : {
        default: e
      };
    }

    var n = r(t);

    function i(e) {
      return null != e && "object" == typeof e && !0 === e["@@functional/placeholder"];
    }

    function o(e) {
      return function t(r) {
        return 0 === arguments.length || i(r) ? t : e.apply(this, arguments);
      };
    }

    function s(e) {
      return function t(r, n) {
        switch (arguments.length) {
          case 0:
            return t;

          case 1:
            return i(r) ? t : o(function (t) {
              return e(r, t);
            });

          default:
            return i(r) && i(n) ? t : i(r) ? o(function (t) {
              return e(t, n);
            }) : i(n) ? o(function (t) {
              return e(r, t);
            }) : e(r, n);
        }
      };
    }

    function a(e, t) {
      var r;
      t = t || [];
      var n = (e = e || []).length,
          i = t.length,
          o = [];

      for (r = 0; r < n;) o[o.length] = e[r], r += 1;

      for (r = 0; r < i;) o[o.length] = t[r], r += 1;

      return o;
    }

    function c(e, t) {
      switch (e) {
        case 0:
          return function () {
            return t.apply(this, arguments);
          };

        case 1:
          return function (e) {
            return t.apply(this, arguments);
          };

        case 2:
          return function (e, r) {
            return t.apply(this, arguments);
          };

        case 3:
          return function (e, r, n) {
            return t.apply(this, arguments);
          };

        case 4:
          return function (e, r, n, i) {
            return t.apply(this, arguments);
          };

        case 5:
          return function (e, r, n, i, o) {
            return t.apply(this, arguments);
          };

        case 6:
          return function (e, r, n, i, o, s) {
            return t.apply(this, arguments);
          };

        case 7:
          return function (e, r, n, i, o, s, a) {
            return t.apply(this, arguments);
          };

        case 8:
          return function (e, r, n, i, o, s, a, c) {
            return t.apply(this, arguments);
          };

        case 9:
          return function (e, r, n, i, o, s, a, c, u) {
            return t.apply(this, arguments);
          };

        case 10:
          return function (e, r, n, i, o, s, a, c, u, f) {
            return t.apply(this, arguments);
          };

        default:
          throw new Error("First argument to _arity must be a non-negative integer no greater than ten");
      }
    }

    function u(e, t, r) {
      return function () {
        for (var n = [], o = 0, s = e, a = 0; a < t.length || o < arguments.length;) {
          var f;
          a < t.length && (!i(t[a]) || o >= arguments.length) ? f = t[a] : (f = arguments[o], o += 1), n[a] = f, i(f) || (s -= 1), a += 1;
        }

        return s <= 0 ? r.apply(this, n) : c(s, u(e, n, r));
      };
    }

    var f = s(function (e, t) {
      return 1 === e ? o(t) : c(e, u(e, [], t));
    });

    function l(e) {
      return function t(r, n, a) {
        switch (arguments.length) {
          case 0:
            return t;

          case 1:
            return i(r) ? t : s(function (t, n) {
              return e(r, t, n);
            });

          case 2:
            return i(r) && i(n) ? t : i(r) ? s(function (t, r) {
              return e(t, n, r);
            }) : i(n) ? s(function (t, n) {
              return e(r, t, n);
            }) : o(function (t) {
              return e(r, n, t);
            });

          default:
            return i(r) && i(n) && i(a) ? t : i(r) && i(n) ? s(function (t, r) {
              return e(t, r, a);
            }) : i(r) && i(a) ? s(function (t, r) {
              return e(t, n, r);
            }) : i(n) && i(a) ? s(function (t, n) {
              return e(r, t, n);
            }) : i(r) ? o(function (t) {
              return e(t, n, a);
            }) : i(n) ? o(function (t) {
              return e(r, t, a);
            }) : i(a) ? o(function (t) {
              return e(r, n, t);
            }) : e(r, n, a);
        }
      };
    }

    var h = Array.isArray || function (e) {
      return null != e && e.length >= 0 && "[object Array]" === Object.prototype.toString.call(e);
    };

    function m(e) {
      return null != e && "function" == typeof e["@@transducer/step"];
    }

    function d(e, t, r) {
      return function () {
        if (0 === arguments.length) return r();
        var n = Array.prototype.slice.call(arguments, 0),
            i = n.pop();

        if (!h(i)) {
          for (var o = 0; o < e.length;) {
            if ("function" == typeof i[e[o]]) return i[e[o]].apply(i, n);
            o += 1;
          }

          if (m(i)) {
            var s = t.apply(null, n);
            return s(i);
          }
        }

        return r.apply(this, arguments);
      };
    }

    var p = function () {
      return this.xf["@@transducer/init"]();
    },
        g = function (e) {
      return this.xf["@@transducer/result"](e);
    };

    function y(e, t) {
      for (var r = 0, n = t.length, i = Array(n); r < n;) i[r] = e(t[r]), r += 1;

      return i;
    }

    function w(e) {
      return "[object String]" === Object.prototype.toString.call(e);
    }

    var x = o(function (e) {
      return !!h(e) || !!e && "object" == typeof e && !w(e) && (1 === e.nodeType ? !!e.length : 0 === e.length || e.length > 0 && e.hasOwnProperty(0) && e.hasOwnProperty(e.length - 1));
    }),
        b = function () {
      function e(e) {
        this.f = e;
      }

      return e.prototype["@@transducer/init"] = function () {
        throw new Error("init not implemented on XWrap");
      }, e.prototype["@@transducer/result"] = function (e) {
        return e;
      }, e.prototype["@@transducer/step"] = function (e, t) {
        return this.f(e, t);
      }, e;
    }();

    var v = s(function (e, t) {
      return c(e.length, function () {
        return e.apply(t, arguments);
      });
    }),
        k = v;

    function S(e, t, r) {
      for (var n = r.next(); !n.done;) {
        if ((t = e["@@transducer/step"](t, n.value)) && t["@@transducer/reduced"]) {
          t = t["@@transducer/value"];
          break;
        }

        n = r.next();
      }

      return e["@@transducer/result"](t);
    }

    function E(e, t, r, n) {
      return e["@@transducer/result"](r[n](k(e["@@transducer/step"], e), t));
    }

    var T = "undefined" != typeof Symbol ? Symbol.iterator : "@@iterator";

    function O(e, t, r) {
      if ("function" == typeof e && (e = function (e) {
        return new b(e);
      }(e)), x(r)) return function (e, t, r) {
        for (var n = 0, i = r.length; n < i;) {
          if ((t = e["@@transducer/step"](t, r[n])) && t["@@transducer/reduced"]) {
            t = t["@@transducer/value"];
            break;
          }

          n += 1;
        }

        return e["@@transducer/result"](t);
      }(e, t, r);
      if ("function" == typeof r["fantasy-land/reduce"]) return E(e, t, r, "fantasy-land/reduce");
      if (null != r[T]) return S(e, t, r[T]());
      if ("function" == typeof r.next) return S(e, t, r);
      if ("function" == typeof r.reduce) return E(e, t, r, "reduce");
      throw new TypeError("reduce: list must be array or iterable");
    }

    var N = function () {
      function e(e, t) {
        this.xf = t, this.f = e;
      }

      return e.prototype["@@transducer/init"] = p, e.prototype["@@transducer/result"] = g, e.prototype["@@transducer/step"] = function (e, t) {
        return this.xf["@@transducer/step"](e, this.f(t));
      }, e;
    }(),
        I = s(function (e, t) {
      return new N(e, t);
    });

    function j(e, t) {
      return Object.prototype.hasOwnProperty.call(t, e);
    }

    var _ = Object.prototype.toString,
        B = function () {
      return "[object Arguments]" === _.call(arguments) ? function (e) {
        return "[object Arguments]" === _.call(e);
      } : function (e) {
        return j("callee", e);
      };
    }(),
        M = B,
        A = !{
      toString: null
    }.propertyIsEnumerable("toString"),
        F = ["constructor", "valueOf", "isPrototypeOf", "toString", "propertyIsEnumerable", "hasOwnProperty", "toLocaleString"],
        z = function () {
      return arguments.propertyIsEnumerable("length");
    }(),
        U = function (e, t) {
      for (var r = 0; r < e.length;) {
        if (e[r] === t) return !0;
        r += 1;
      }

      return !1;
    },
        L = "function" != typeof Object.keys || z ? o(function (e) {
      if (Object(e) !== e) return [];
      var t,
          r,
          n = [],
          i = z && M(e);

      for (t in e) !j(t, e) || i && "length" === t || (n[n.length] = t);

      if (A) for (r = F.length - 1; r >= 0;) j(t = F[r], e) && !U(n, t) && (n[n.length] = t), r -= 1;
      return n;
    }) : o(function (e) {
      return Object(e) !== e ? [] : Object.keys(e);
    }),
        $ = L,
        P = s(d(["fantasy-land/map", "map"], I, function (e, t) {
      switch (Object.prototype.toString.call(t)) {
        case "[object Function]":
          return f(t.length, function () {
            return e.call(this, t.apply(this, arguments));
          });

        case "[object Object]":
          return O(function (r, n) {
            return r[n] = e(t[n]), r;
          }, {}, $(t));

        default:
          return y(e, t);
      }
    })),
        D = P,
        R = Number.isInteger || function (e) {
      return e << 0 === e;
    },
        C = s(function (e, t) {
      var r = e < 0 ? t.length + e : e;
      return w(t) ? t.charAt(r) : t[r];
    }),
        q = s(function (e, t) {
      return e.map(function (e) {
        for (var r, n = t, i = 0; i < e.length;) {
          if (null == n) return;
          r = e[i], n = R(r) ? C(r, n) : n[r], i += 1;
        }

        return n;
      });
    }),
        H = s(function (e, t) {
      return q([e], t)[0];
    }),
        W = s(function (e, t) {
      return H([e], t);
    }),
        Y = l(O),
        V = s(function (e, t) {
      return "function" == typeof t["fantasy-land/ap"] ? t["fantasy-land/ap"](e) : "function" == typeof e.ap ? e.ap(t) : "function" == typeof e ? function (r) {
        return e(r)(t(r));
      } : O(function (e, r) {
        return a(e, D(r, t));
      }, [], e);
    }),
        Q = o(function (e) {
      for (var t = $(e), r = t.length, n = [], i = 0; i < r;) n[i] = e[t[i]], i += 1;

      return n;
    }),
        Z = o(function (e) {
      return null == e;
    });

    function G(e) {
      var t = Object.prototype.toString.call(e);
      return "[object Function]" === t || "[object AsyncFunction]" === t || "[object GeneratorFunction]" === t || "[object AsyncGeneratorFunction]" === t;
    }

    var J = s(function (e, t) {
      var r = f(e, t);
      return f(e, function () {
        return O(V, D(r, arguments[0]), Array.prototype.slice.call(arguments, 1));
      });
    }),
        X = J,
        K = o(function (e) {
      return X(e.length, e);
    }),
        ee = o(function (e) {
      return f(e.length, e);
    });

    function te(e) {
      return function t(r) {
        for (var n, i, o, s = [], a = 0, c = r.length; a < c;) {
          if (x(r[a])) for (o = 0, i = (n = e ? t(r[a]) : r[a]).length; o < i;) s[s.length] = n[o], o += 1;else s[s.length] = r[a];
          a += 1;
        }

        return s;
      };
    }

    var re = o(function (e) {
      return null === e ? "Null" : void 0 === e ? "Undefined" : Object.prototype.toString.call(e).slice(8, -1);
    }),
        ne = K(o(function (e) {
      return !e;
    }));

    function ie(e, t) {
      return function () {
        return t.call(this, e.apply(this, arguments));
      };
    }

    function oe(e, t) {
      return function () {
        var r = arguments.length;
        if (0 === r) return t();
        var n = arguments[r - 1];
        return h(n) || "function" != typeof n[e] ? t.apply(this, arguments) : n[e].apply(n, Array.prototype.slice.call(arguments, 0, r - 1));
      };
    }

    var se = l(oe("slice", function (e, t, r) {
      return Array.prototype.slice.call(r, e, t);
    })),
        ae = o(oe("tail", se(1, 1 / 0)));

    function ce() {
      if (0 === arguments.length) throw new Error("pipe requires at least one argument");
      return c(arguments[0].length, Y(ie, arguments[0], ae(arguments)));
    }

    var ue = o(function (e) {
      return w(e) ? e.split("").reverse().join("") : Array.prototype.slice.call(e, 0).reverse();
    });

    function fe() {
      if (0 === arguments.length) throw new Error("compose requires at least one argument");
      return ce.apply(this, ue(arguments));
    }

    function le(e) {
      return e;
    }

    var he = o(le);

    function me(e) {
      for (var t, r = []; !(t = e.next()).done;) r.push(t.value);

      return r;
    }

    function de(e, t, r) {
      for (var n = 0, i = r.length; n < i;) {
        if (e(t, r[n])) return !0;
        n += 1;
      }

      return !1;
    }

    var pe = "function" == typeof Object.is ? Object.is : function (e, t) {
      return e === t ? 0 !== e || 1 / e == 1 / t : e != e && t != t;
    };

    function ge(e, t, r, n) {
      var i = me(e);

      function o(e, t) {
        return ye(e, t, r.slice(), n.slice());
      }

      return !de(function (e, t) {
        return !de(o, t, e);
      }, me(t), i);
    }

    function ye(e, t, r, n) {
      if (pe(e, t)) return !0;
      var i,
          o,
          s = re(e);
      if (s !== re(t)) return !1;
      if (null == e || null == t) return !1;
      if ("function" == typeof e["fantasy-land/equals"] || "function" == typeof t["fantasy-land/equals"]) return "function" == typeof e["fantasy-land/equals"] && e["fantasy-land/equals"](t) && "function" == typeof t["fantasy-land/equals"] && t["fantasy-land/equals"](e);
      if ("function" == typeof e.equals || "function" == typeof t.equals) return "function" == typeof e.equals && e.equals(t) && "function" == typeof t.equals && t.equals(e);

      switch (s) {
        case "Arguments":
        case "Array":
        case "Object":
          if ("function" == typeof e.constructor && "Promise" === (i = e.constructor, null == (o = String(i).match(/^function (\w*)/)) ? "" : o[1])) return e === t;
          break;

        case "Boolean":
        case "Number":
        case "String":
          if (typeof e != typeof t || !pe(e.valueOf(), t.valueOf())) return !1;
          break;

        case "Date":
          if (!pe(e.valueOf(), t.valueOf())) return !1;
          break;

        case "Error":
          return e.name === t.name && e.message === t.message;

        case "RegExp":
          if (e.source !== t.source || e.global !== t.global || e.ignoreCase !== t.ignoreCase || e.multiline !== t.multiline || e.sticky !== t.sticky || e.unicode !== t.unicode) return !1;
      }

      for (var a = r.length - 1; a >= 0;) {
        if (r[a] === e) return n[a] === t;
        a -= 1;
      }

      switch (s) {
        case "Map":
          return e.size === t.size && ge(e.entries(), t.entries(), r.concat([e]), n.concat([t]));

        case "Set":
          return e.size === t.size && ge(e.values(), t.values(), r.concat([e]), n.concat([t]));

        case "Arguments":
        case "Array":
        case "Object":
        case "Boolean":
        case "Number":
        case "String":
        case "Date":
        case "Error":
        case "RegExp":
        case "Int8Array":
        case "Uint8Array":
        case "Uint8ClampedArray":
        case "Int16Array":
        case "Uint16Array":
        case "Int32Array":
        case "Uint32Array":
        case "Float32Array":
        case "Float64Array":
        case "ArrayBuffer":
          break;

        default:
          return !1;
      }

      var c = $(e);
      if (c.length !== $(t).length) return !1;
      var u = r.concat([e]),
          f = n.concat([t]);

      for (a = c.length - 1; a >= 0;) {
        var l = c[a];
        if (!j(l, t) || !ye(t[l], e[l], u, f)) return !1;
        a -= 1;
      }

      return !0;
    }

    var we = s(function (e, t) {
      return ye(e, t, [], []);
    });

    function xe(e, t) {
      return function (e, t, r) {
        var n, i;
        if ("function" == typeof e.indexOf) switch (typeof t) {
          case "number":
            if (0 === t) {
              for (n = 1 / t; r < e.length;) {
                if (0 === (i = e[r]) && 1 / i === n) return r;
                r += 1;
              }

              return -1;
            }

            if (t != t) {
              for (; r < e.length;) {
                if ("number" == typeof (i = e[r]) && i != i) return r;
                r += 1;
              }

              return -1;
            }

            return e.indexOf(t, r);

          case "string":
          case "boolean":
          case "function":
          case "undefined":
            return e.indexOf(t, r);

          case "object":
            if (null === t) return e.indexOf(t, r);
        }

        for (; r < e.length;) {
          if (we(e[r], t)) return r;
          r += 1;
        }

        return -1;
      }(t, e, 0) >= 0;
    }

    function be(e) {
      return '"' + e.replace(/\\/g, "\\\\").replace(/[\b]/g, "\\b").replace(/\f/g, "\\f").replace(/\n/g, "\\n").replace(/\r/g, "\\r").replace(/\t/g, "\\t").replace(/\v/g, "\\v").replace(/\0/g, "\\0").replace(/"/g, '\\"') + '"';
    }

    var ve = function (e) {
      return (e < 10 ? "0" : "") + e;
    },
        ke = "function" == typeof Date.prototype.toISOString ? function (e) {
      return e.toISOString();
    } : function (e) {
      return e.getUTCFullYear() + "-" + ve(e.getUTCMonth() + 1) + "-" + ve(e.getUTCDate()) + "T" + ve(e.getUTCHours()) + ":" + ve(e.getUTCMinutes()) + ":" + ve(e.getUTCSeconds()) + "." + (e.getUTCMilliseconds() / 1e3).toFixed(3).slice(2, 5) + "Z";
    };

    function Se(e) {
      return "[object Object]" === Object.prototype.toString.call(e);
    }

    var Ee = function () {
      function e(e, t) {
        this.xf = t, this.f = e;
      }

      return e.prototype["@@transducer/init"] = p, e.prototype["@@transducer/result"] = g, e.prototype["@@transducer/step"] = function (e, t) {
        return this.f(t) ? this.xf["@@transducer/step"](e, t) : e;
      }, e;
    }(),
        Te = s(d(["filter"], s(function (e, t) {
      return new Ee(e, t);
    }), function (e, t) {
      return Se(t) ? O(function (r, n) {
        return e(t[n]) && (r[n] = t[n]), r;
      }, {}, $(t)) : function (e, t) {
        for (var r = 0, n = t.length, i = []; r < n;) e(t[r]) && (i[i.length] = t[r]), r += 1;

        return i;
      }(e, t);
    })),
        Oe = s(function (e, t) {
      return Te((r = e, function () {
        return !r.apply(this, arguments);
      }), t);
      var r;
    }),
        Ne = Oe;

    function Ie(e, t) {
      var r = function (r) {
        var n = t.concat([e]);
        return xe(r, n) ? "<Circular>" : Ie(r, n);
      },
          n = function (e, t) {
        return y(function (t) {
          return be(t) + ": " + r(e[t]);
        }, t.slice().sort());
      };

      switch (Object.prototype.toString.call(e)) {
        case "[object Arguments]":
          return "(function() { return arguments; }(" + y(r, e).join(", ") + "))";

        case "[object Array]":
          return "[" + y(r, e).concat(n(e, Ne(function (e) {
            return /^\d+$/.test(e);
          }, $(e)))).join(", ") + "]";

        case "[object Boolean]":
          return "object" == typeof e ? "new Boolean(" + r(e.valueOf()) + ")" : e.toString();

        case "[object Date]":
          return "new Date(" + (isNaN(e.valueOf()) ? r(NaN) : be(ke(e))) + ")";

        case "[object Null]":
          return "null";

        case "[object Number]":
          return "object" == typeof e ? "new Number(" + r(e.valueOf()) + ")" : 1 / e == -1 / 0 ? "-0" : e.toString(10);

        case "[object String]":
          return "object" == typeof e ? "new String(" + r(e.valueOf()) + ")" : be(e);

        case "[object Undefined]":
          return "undefined";

        default:
          if ("function" == typeof e.toString) {
            var i = e.toString();
            if ("[object Object]" !== i) return i;
          }

          return "{" + n(e, $(e)).join(", ") + "}";
      }
    }

    var je = o(function (e) {
      return Ie(e, []);
    }),
        _e = s(function (e, t) {
      if (h(e)) {
        if (h(t)) return e.concat(t);
        throw new TypeError(je(t) + " is not an array");
      }

      if (w(e)) {
        if (w(t)) return e + t;
        throw new TypeError(je(t) + " is not a string");
      }

      if (null != e && G(e["fantasy-land/concat"])) return e["fantasy-land/concat"](t);
      if (null != e && G(e.concat)) return e.concat(t);
      throw new TypeError(je(e) + ' does not have a method named "concat" or "fantasy-land/concat"');
    });

    function Be(e, t, r) {
      var n,
          i = typeof e;

      switch (i) {
        case "string":
        case "number":
          return 0 === e && 1 / e == -1 / 0 ? !!r._items["-0"] || (t && (r._items["-0"] = !0), !1) : null !== r._nativeSet ? t ? (n = r._nativeSet.size, r._nativeSet.add(e), r._nativeSet.size === n) : r._nativeSet.has(e) : i in r._items ? e in r._items[i] || (t && (r._items[i][e] = !0), !1) : (t && (r._items[i] = {}, r._items[i][e] = !0), !1);

        case "boolean":
          if (i in r._items) {
            var o = e ? 1 : 0;
            return !!r._items[i][o] || (t && (r._items[i][o] = !0), !1);
          }

          return t && (r._items[i] = e ? [!1, !0] : [!0, !1]), !1;

        case "function":
          return null !== r._nativeSet ? t ? (n = r._nativeSet.size, r._nativeSet.add(e), r._nativeSet.size === n) : r._nativeSet.has(e) : i in r._items ? !!xe(e, r._items[i]) || (t && r._items[i].push(e), !1) : (t && (r._items[i] = [e]), !1);

        case "undefined":
          return !!r._items[i] || (t && (r._items[i] = !0), !1);

        case "object":
          if (null === e) return !!r._items.null || (t && (r._items.null = !0), !1);

        default:
          return (i = Object.prototype.toString.call(e)) in r._items ? !!xe(e, r._items[i]) || (t && r._items[i].push(e), !1) : (t && (r._items[i] = [e]), !1);
      }
    }

    var Me = function () {
      function e() {
        this._nativeSet = "function" == typeof Set ? new Set() : null, this._items = {};
      }

      return e.prototype.add = function (e) {
        return !Be(e, !0, this);
      }, e.prototype.has = function (e) {
        return Be(e, !1, this);
      }, e;
    }(),
        Ae = o(function (e) {
      return null != e && "function" == typeof e["fantasy-land/empty"] ? e["fantasy-land/empty"]() : null != e && null != e.constructor && "function" == typeof e.constructor["fantasy-land/empty"] ? e.constructor["fantasy-land/empty"]() : null != e && "function" == typeof e.empty ? e.empty() : null != e && null != e.constructor && "function" == typeof e.constructor.empty ? e.constructor.empty() : h(e) ? [] : w(e) ? "" : Se(e) ? {} : M(e) ? function () {
        return arguments;
      }() : void 0;
    }),
        Fe = Ae,
        ze = function () {
      function e(e, t) {
        this.xf = t, this.f = e, this.found = !1;
      }

      return e.prototype["@@transducer/init"] = p, e.prototype["@@transducer/result"] = function (e) {
        return this.found || (e = this.xf["@@transducer/step"](e, void 0)), this.xf["@@transducer/result"](e);
      }, e.prototype["@@transducer/step"] = function (e, t) {
        var r;
        return this.f(t) && (this.found = !0, e = (r = this.xf["@@transducer/step"](e, t)) && r["@@transducer/reduced"] ? r : {
          "@@transducer/value": r,
          "@@transducer/reduced": !0
        }), e;
      }, e;
    }(),
        Ue = s(d(["find"], s(function (e, t) {
      return new ze(e, t);
    }), function (e, t) {
      for (var r = 0, n = t.length; r < n;) {
        if (e(t[r])) return t[r];
        r += 1;
      }
    })),
        Le = o(te(!0)),
        $e = o(function (e) {
      for (var t = {}, r = 0; r < e.length;) t[e[r][0]] = e[r][1], r += 1;

      return t;
    }),
        Pe = s(function (e, t) {
      if (0 === e.length || Z(t)) return !1;

      for (var r = t, n = 0; n < e.length;) {
        if (Z(r) || !j(e[n], r)) return !1;
        r = r[e[n]], n += 1;
      }

      return !0;
    }),
        De = s(function (e, t) {
      for (var r, n, i = new Me(), o = [], s = 0; s < t.length;) r = e(n = t[s]), i.add(r) && o.push(n), s += 1;

      return o;
    })(he);

    var Re = "function" == typeof Object.assign ? Object.assign : function (e) {
      if (null == e) throw new TypeError("Cannot convert undefined or null to object");

      for (var t = Object(e), r = 1, n = arguments.length; r < n;) {
        var i = arguments[r];
        if (null != i) for (var o in i) j(o, i) && (t[o] = i[o]);
        r += 1;
      }

      return t;
    },
        Ce = s(function (e, t) {
      return f(e + 1, function () {
        var r = arguments[e];
        if (null != r && G(r[t])) return r[t].apply(r, Array.prototype.slice.call(arguments, 0, e));
        throw new TypeError(je(r) + ' does not have a method named "' + t + '"');
      });
    }),
        qe = Ce,
        He = s(function (e, t) {
      return null != t && t.constructor === e || t instanceof e;
    }),
        We = o(function (e) {
      return null != e && we(e, Fe(e));
    }),
        Ye = o(function (e) {
      return Re.apply(null, [{}].concat(e));
    }),
        Ve = s(function (e, t) {
      for (var r = {}, n = {}, i = 0, o = e.length; i < o;) n[e[i]] = 1, i += 1;

      for (var s in t) n.hasOwnProperty(s) || (r[s] = t[s]);

      return r;
    }),
        Qe = Ve,
        Ze = l(function (e, t, r) {
      return we(t, r[e]);
    }),
        Ge = l(function (e, t, r) {
      return r.replace(e, t);
    }),
        Je = qe(1, "split"),
        Xe = "\t\n\v\f\r                　\u2028\u2029\ufeff",
        Ke = o("function" == typeof String.prototype.trim && !Xe.trim() && "​".trim() ? function (e) {
      return e.trim();
    } : function (e) {
      var t = new RegExp("^[" + Xe + "][" + Xe + "]*"),
          r = new RegExp("[" + Xe + "][" + Xe + "]*$");
      return e.replace(t, "").replace(r, "");
    }),
        et = s(fe(De, a));

    const tt = e => {
      if (Z(e) || We(e)) return e;

      switch (re(e)) {
        case "String":
          return Ke(e);

        case "Array":
          return Te(ne(We))(e);

        default:
          return e;
      }
    },
          rt = (e, t) => We(t) || Z(t) ? [] : ce(Je(e), tt)(t),
          nt = {
      set: (e, t) => {
        "undefined" != typeof window && window.sessionStorage.setItem(e, t);
      },
      get: e => "undefined" != typeof window ? window.sessionStorage.getItem(e) : null,
      remove: e => {
        "undefined" != typeof window && window.sessionStorage.removeItem(e);
      },
      clear: () => {
        "undefined" != typeof window && window.sessionStorage.clear();
      }
    },
          it = {
      set: (e, t) => {
        "undefined" != typeof window && window.localStorage.setItem(e, t);
      },
      get: e => "undefined" != typeof window ? window.localStorage.getItem(e) : null,
      remove: e => {
        "undefined" != typeof window && window.localStorage.removeItem(e);
      },
      clear: () => {
        "undefined" != typeof window && window.localStorage.clear();
      }
    },
          ot = {
      set: (e, t, r) => {
        const n = new Date();
        n.setTime(n.getTime() + 24 * r * 60 * 60 * 1e3);
        const i = `expires=${n.toUTCString()}`;
        document.cookie = `${e}=${escape(t)};${i};path=/`;
      },
      get: e => {
        const t = `${e}=`,
              r = document.cookie.split(";");

        for (let e = 0; e < r.length; e += 1) {
          let n = r[e];

          for (; " " === n.charAt(0);) n = n.substring(1);

          if (-1 !== n.indexOf(t)) return n.substring(t.length, n.length);
        }
      }
    };

    function st(e, t, r, n) {
      e.addEventListener(t, function n(i) {
        e.removeEventListener(t, n), r(i);
      }, n);
    }

    const at = ["id", "type", "async", "defer", "crossOrigin", "integrity", "noModule"];

    const ct = e => {
      let t = e;
      return e instanceof Date ? t = e : "string" == typeof e && String(e).length >= 13 && !isNaN(e) ? t = Number(e) : "string" == typeof e && e.includes("-") && (t = e.replace(/-/g, "/")), new Date(t) && "Invalid Date" !== String(new Date(t));
    },
          ut = e => {
      const t = e.replace(/-/g, "/");
      return new Date(t).valueOf();
    },
          ft = (e, t) => {
      if (!ct(e)) return null;
      let r;
      isNaN(e) ? "string" == typeof e && e.includes("-") && (e = e.replace(/-/g, "/")) : e = Number(e), r = new Date(e);
      const n = {
        YYYY: r.getFullYear(),
        YY: `${r.getFullYear()}`.slice(-2),
        MM: `0${r.getMonth() + 1}`.slice(-2),
        M: r.getMonth() + 1,
        DD: `0${r.getDate()}`.slice(-2),
        D: r.getDate(),
        HH: `0${r.getHours()}`.slice(-2),
        H: r.getHours(),
        hh: ("0" + r.getHours() % 12).slice(-2),
        h: r.getHours() % 12,
        mm: `0${r.getMinutes()}`.slice(-2),
        m: r.getMinutes(),
        ss: `0${r.getSeconds()}`.slice(-2),
        s: r.getSeconds(),
        w: ["日", "一", "二", "三", "四", "五", "六"][r.getDay()]
      };

      for (const e in n) t = t.replace(e, n[e]);

      return t;
    },
          lt = (e, t, r, n) => {
      let i = "";
      return t.length > r ? t : (i = "left" === e ? new Array(r - t.length + 1).join(n) + t : t + new Array(r - t.length + 1).join(n), i);
    },
          [ht, mt, dt] = [86400, 3600, 60],
          pt = (e, t, r = !0) => {
      let n = e < 0 ? 0 : parseFloat(`${e}`);
      if ((isNaN(n) || !n) && (!r || 0 !== n)) return "-";
      let [i, o, s, a] = [void 0, void 0, void 0, void 0];
      t.indexOf("{DD}") > -1 && (i = Math.floor(n / ht), n = Math.floor(n % ht)), t.indexOf("{HH}") > -1 && (o = Math.floor(n / mt), n = Math.floor(n % mt)), t.indexOf("{mm}") > -1 && (s = Math.floor(n / dt), n = Math.floor(n % dt)), a = n;
      return t.replace("{ss}", "number" == typeof a ? lt("left", String(a), 2, "0") : "").replace("{mm}", "number" == typeof s ? lt("left", String(s), 2, "0") : "").replace("{HH}", "number" == typeof o ? lt("left", String(o), 2, "0") : "").replace("{DD}", "number" == typeof i ? lt("left", String(i), 2, "0") : "");
    },
          [gt, yt, wt] = [86400, 3600, 60],
          xt = (e, t, r) => {
      const n = {
        reduce: ee((e, r, i) => {
          const o = e(r, i);
          return (e => Pe([t], e))(i) && i[t] ? i[t].reduce(n.reduce(e), o) : o;
        }),
        find: ee((e, t, r) => ce(D(n.reduce(e, t)), Le)(r))
      },
            i = (r, n) => e.value && n[e.key] && n[e.key] === e.value || !e.value && n[e.key] ? r.concat([Qe([t], n)]) : r;

      switch (re(r)) {
        case "Object":
          return n.reduce(i, [])(r);

        case "Array":
          return n.find(i, [])(r);

        default:
          return r;
      }
    },
          bt = (e, t) => {
      let r = [],
          n = [],
          i = {};
      const o = Ue(Ze("origion_url", `/${e[0]}`))(t);

      if (o) {
        if (r = [o.name], e.length > 1 && o.children && o.children.length) {
          const t = e.join("/");
          i = Ue(Ze("origion_url", `/${t}`))(o.children);
        } else i = o.children && o.children.length ? o.children[0] : o;

        n = (null == i ? void 0 : i.name) ? [i.name] : [];
      }

      return {
        defaultOpenMenus: r,
        defaultSelectedMenus: n,
        defaultMenu: i
      };
    },
          vt = e => {
      var t, r;
      let n = [],
          i = [],
          o = {};
      return e[0] && e[0].children && e[0].children.length ? (n = [e[0].name], i = [e[0].children[0].name], o = e[0].children[0]) : e[0] && (n = [null === (t = e[0]) || void 0 === t ? void 0 : t.name], i = [null === (r = e[0]) || void 0 === r ? void 0 : r.name], o = e[0]), {
        defaultOpenMenus: n,
        defaultSelectedMenus: i,
        defaultMenu: o
      };
    },
          kt = (e, t = 0, r = !0, n = !1) => {
      let i = parseFloat(e);
      if (isNaN(i) || 0 !== e && !e) return "-";
      let o = !1,
          s = "";
      return i < 0 && (o = !0, i = Math.abs(i)), i >= 1e8 ? s = (i / 1e8).toFixed(t) + (r ? "亿" : "") : i >= 1e4 ? s = (i / 1e4).toFixed(t) + (r ? "万" : "") : (s = n ? i.toFixed(0) : i.toString(), s = n || isNaN(t) ? s : i.toFixed(t)), o ? `-${s}` : s;
    },
          St = () => window.localStorage.getItem("env") || "prod",
          Et = e => {
      if (!e) return {};
      return ce(Ge("?", ""), Je("&"), D(e => Je("=", decodeURI(e))), $e)(e);
    };

    var Tt = Object.freeze({
      __proto__: null,
      parse: Et,
      stringify: e => {
        if (We(e)) return "";
        const t = Object.keys(e);
        let r = "";
        t.map(t => {
          r += `&${t}=${e[t]}`;
        });
        return new URLSearchParams(r).toString() || "";
      }
    });

    var Ot = 1e6,
        Nt = 1e6,
        It = "[big.js] ",
        jt = It + "Invalid ",
        _t = jt + "decimal places",
        Bt = jt + "rounding mode",
        Mt = It + "Division by zero",
        At = {},
        Ft = void 0,
        zt = /^-?(\d+(\.\d*)?|\.\d+)(e[+-]?\d+)?$/i;

    function Ut(e, t, r, n) {
      var i = e.c,
          o = e.e + t + 1;

      if (o < i.length) {
        if (1 === r) n = i[o] >= 5;else if (2 === r) n = i[o] > 5 || 5 == i[o] && (n || o < 0 || i[o + 1] !== Ft || 1 & i[o - 1]);else if (3 === r) n = n || !!i[0];else if (n = !1, 0 !== r) throw Error(Bt);
        if (o < 1) i.length = 1, n ? (e.e = -t, i[0] = 1) : i[0] = e.e = 0;else {
          if (i.length = o--, n) for (; ++i[o] > 9;) i[o] = 0, o-- || (++e.e, i.unshift(1));

          for (o = i.length; !i[--o];) i.pop();
        }
      } else if (r < 0 || r > 3 || r !== ~~r) throw Error(Bt);

      return e;
    }

    function Lt(e, t, r, n) {
      var i,
          o,
          s = e.constructor,
          a = !e.c[0];

      if (r !== Ft) {
        if (r !== ~~r || r < (3 == t) || r > Ot) throw Error(3 == t ? jt + "precision" : _t);

        for (r = n - (e = new s(e)).e, e.c.length > ++n && Ut(e, r, s.RM), 2 == t && (n = e.e + r + 1); e.c.length < n;) e.c.push(0);
      }

      if (i = e.e, r = (o = e.c.join("")).length, 2 != t && (1 == t || 3 == t && n <= i || i <= s.NE || i >= s.PE)) o = o.charAt(0) + (r > 1 ? "." + o.slice(1) : "") + (i < 0 ? "e" : "e+") + i;else if (i < 0) {
        for (; ++i;) o = "0" + o;

        o = "0." + o;
      } else if (i > 0) {
        if (++i > r) for (i -= r; i--;) o += "0";else i < r && (o = o.slice(0, i) + "." + o.slice(i));
      } else r > 1 && (o = o.charAt(0) + "." + o.slice(1));
      return e.s < 0 && (!a || 4 == t) ? "-" + o : o;
    }

    At.abs = function () {
      var e = new this.constructor(this);
      return e.s = 1, e;
    }, At.cmp = function (e) {
      var t,
          r = this,
          n = r.c,
          i = (e = new r.constructor(e)).c,
          o = r.s,
          s = e.s,
          a = r.e,
          c = e.e;
      if (!n[0] || !i[0]) return n[0] ? o : i[0] ? -s : 0;
      if (o != s) return o;
      if (t = o < 0, a != c) return a > c ^ t ? 1 : -1;

      for (s = (a = n.length) < (c = i.length) ? a : c, o = -1; ++o < s;) if (n[o] != i[o]) return n[o] > i[o] ^ t ? 1 : -1;

      return a == c ? 0 : a > c ^ t ? 1 : -1;
    }, At.div = function (e) {
      var t = this,
          r = t.constructor,
          n = t.c,
          i = (e = new r(e)).c,
          o = t.s == e.s ? 1 : -1,
          s = r.DP;
      if (s !== ~~s || s < 0 || s > Ot) throw Error(_t);
      if (!i[0]) throw Error(Mt);
      if (!n[0]) return new r(0 * o);
      var a,
          c,
          u,
          f,
          l,
          h = i.slice(),
          m = a = i.length,
          d = n.length,
          p = n.slice(0, a),
          g = p.length,
          y = e,
          w = y.c = [],
          x = 0,
          b = s + (y.e = t.e - e.e) + 1;

      for (y.s = o, o = b < 0 ? 0 : b, h.unshift(0); g++ < a;) p.push(0);

      do {
        for (u = 0; u < 10; u++) {
          if (a != (g = p.length)) f = a > g ? 1 : -1;else for (l = -1, f = 0; ++l < a;) if (i[l] != p[l]) {
            f = i[l] > p[l] ? 1 : -1;
            break;
          }
          if (!(f < 0)) break;

          for (c = g == a ? i : h; g;) {
            if (p[--g] < c[g]) {
              for (l = g; l && !p[--l];) p[l] = 9;

              --p[l], p[g] += 10;
            }

            p[g] -= c[g];
          }

          for (; !p[0];) p.shift();
        }

        w[x++] = f ? u : ++u, p[0] && f ? p[g] = n[m] || 0 : p = [n[m]];
      } while ((m++ < d || p[0] !== Ft) && o--);

      return w[0] || 1 == x || (w.shift(), y.e--), x > b && Ut(y, s, r.RM, p[0] !== Ft), y;
    }, At.eq = function (e) {
      return !this.cmp(e);
    }, At.gt = function (e) {
      return this.cmp(e) > 0;
    }, At.gte = function (e) {
      return this.cmp(e) > -1;
    }, At.lt = function (e) {
      return this.cmp(e) < 0;
    }, At.lte = function (e) {
      return this.cmp(e) < 1;
    }, At.minus = At.sub = function (e) {
      var t,
          r,
          n,
          i,
          o = this,
          s = o.constructor,
          a = o.s,
          c = (e = new s(e)).s;
      if (a != c) return e.s = -c, o.plus(e);
      var u = o.c.slice(),
          f = o.e,
          l = e.c,
          h = e.e;
      if (!u[0] || !l[0]) return l[0] ? (e.s = -c, e) : new s(u[0] ? o : 0);

      if (a = f - h) {
        for ((i = a < 0) ? (a = -a, n = u) : (h = f, n = l), n.reverse(), c = a; c--;) n.push(0);

        n.reverse();
      } else for (r = ((i = u.length < l.length) ? u : l).length, a = c = 0; c < r; c++) if (u[c] != l[c]) {
        i = u[c] < l[c];
        break;
      }

      if (i && (n = u, u = l, l = n, e.s = -e.s), (c = (r = l.length) - (t = u.length)) > 0) for (; c--;) u[t++] = 0;

      for (c = t; r > a;) {
        if (u[--r] < l[r]) {
          for (t = r; t && !u[--t];) u[t] = 9;

          --u[t], u[r] += 10;
        }

        u[r] -= l[r];
      }

      for (; 0 === u[--c];) u.pop();

      for (; 0 === u[0];) u.shift(), --h;

      return u[0] || (e.s = 1, u = [h = 0]), e.c = u, e.e = h, e;
    }, At.mod = function (e) {
      var t,
          r = this,
          n = r.constructor,
          i = r.s,
          o = (e = new n(e)).s;
      if (!e.c[0]) throw Error(Mt);
      return r.s = e.s = 1, t = 1 == e.cmp(r), r.s = i, e.s = o, t ? new n(r) : (i = n.DP, o = n.RM, n.DP = n.RM = 0, r = r.div(e), n.DP = i, n.RM = o, this.minus(r.times(e)));
    }, At.plus = At.add = function (e) {
      var t,
          r = this,
          n = r.constructor,
          i = r.s,
          o = (e = new n(e)).s;
      if (i != o) return e.s = -o, r.minus(e);
      var s = r.e,
          a = r.c,
          c = e.e,
          u = e.c;
      if (!a[0] || !u[0]) return u[0] ? e : new n(a[0] ? r : 0 * i);

      if (a = a.slice(), i = s - c) {
        for (i > 0 ? (c = s, t = u) : (i = -i, t = a), t.reverse(); i--;) t.push(0);

        t.reverse();
      }

      for (a.length - u.length < 0 && (t = u, u = a, a = t), i = u.length, o = 0; i; a[i] %= 10) o = (a[--i] = a[i] + u[i] + o) / 10 | 0;

      for (o && (a.unshift(o), ++c), i = a.length; 0 === a[--i];) a.pop();

      return e.c = a, e.e = c, e;
    }, At.pow = function (e) {
      var t = this,
          r = new t.constructor(1),
          n = r,
          i = e < 0;
      if (e !== ~~e || e < -1e6 || e > Nt) throw Error(jt + "exponent");

      for (i && (e = -e); 1 & e && (n = n.times(t)), e >>= 1;) t = t.times(t);

      return i ? r.div(n) : n;
    }, At.round = function (e, t) {
      var r = this.constructor;
      if (e === Ft) e = 0;else if (e !== ~~e || e < -Ot || e > Ot) throw Error(_t);
      return Ut(new r(this), e, t === Ft ? r.RM : t);
    }, At.sqrt = function () {
      var e,
          t,
          r,
          n = this,
          i = n.constructor,
          o = n.s,
          s = n.e,
          a = new i(.5);
      if (!n.c[0]) return new i(n);
      if (o < 0) throw Error(It + "No square root");
      0 === (o = Math.sqrt(n + "")) || o === 1 / 0 ? ((t = n.c.join("")).length + s & 1 || (t += "0"), s = ((s + 1) / 2 | 0) - (s < 0 || 1 & s), e = new i(((o = Math.sqrt(t)) == 1 / 0 ? "1e" : (o = o.toExponential()).slice(0, o.indexOf("e") + 1)) + s)) : e = new i(o), s = e.e + (i.DP += 4);

      do {
        r = e, e = a.times(r.plus(n.div(r)));
      } while (r.c.slice(0, s).join("") !== e.c.slice(0, s).join(""));

      return Ut(e, i.DP -= 4, i.RM);
    }, At.times = At.mul = function (e) {
      var t,
          r = this,
          n = r.constructor,
          i = r.c,
          o = (e = new n(e)).c,
          s = i.length,
          a = o.length,
          c = r.e,
          u = e.e;
      if (e.s = r.s == e.s ? 1 : -1, !i[0] || !o[0]) return new n(0 * e.s);

      for (e.e = c + u, s < a && (t = i, i = o, o = t, u = s, s = a, a = u), t = new Array(u = s + a); u--;) t[u] = 0;

      for (c = a; c--;) {
        for (a = 0, u = s + c; u > c;) a = t[u] + o[c] * i[u - c - 1] + a, t[u--] = a % 10, a = a / 10 | 0;

        t[u] = (t[u] + a) % 10;
      }

      for (a ? ++e.e : t.shift(), c = t.length; !t[--c];) t.pop();

      return e.c = t, e;
    }, At.toExponential = function (e) {
      return Lt(this, 1, e, e);
    }, At.toFixed = function (e) {
      return Lt(this, 2, e, this.e + e);
    }, At.toPrecision = function (e) {
      return Lt(this, 3, e, e - 1);
    }, At.toString = function () {
      return Lt(this);
    }, At.valueOf = At.toJSON = function () {
      return Lt(this, 4);
    };

    var $t = function e() {
      function t(r) {
        var n = this;
        if (!(n instanceof t)) return r === Ft ? e() : new t(r);
        r instanceof t ? (n.s = r.s, n.e = r.e, n.c = r.c.slice()) : function (e, t) {
          var r, n, i;
          if (0 === t && 1 / t < 0) t = "-0";else if (!zt.test(t += "")) throw Error(jt + "number");
          e.s = "-" == t.charAt(0) ? (t = t.slice(1), -1) : 1, (r = t.indexOf(".")) > -1 && (t = t.replace(".", ""));
          (n = t.search(/e/i)) > 0 ? (r < 0 && (r = n), r += +t.slice(n + 1), t = t.substring(0, n)) : r < 0 && (r = t.length);

          for (i = t.length, n = 0; n < i && "0" == t.charAt(n);) ++n;

          if (n == i) e.c = [e.e = 0];else {
            for (; i > 0 && "0" == t.charAt(--i););

            for (e.e = r - n - 1, e.c = [], r = 0; n <= i;) e.c[r++] = +t.charAt(n++);
          }
        }(n, r), n.constructor = t;
      }

      return t.prototype = At, t.DP = 20, t.RM = 1, t.NE = -7, t.PE = 21, t.version = "5.2.2", t;
    }();

    const Pt = (e, t) => {
      const r = (e = e.toString()).indexOf(".");
      return e = -1 !== r ? e.substring(0, t + r + 1) : e.substring(0), Number(parseFloat(e).toFixed(t));
    },
          Dt = e => (t, r) => {
      if (We(t) || Z(t)) return t;
      if (isNaN(Number(t))) return NaN;
      const n = Number(new $t(Number(t)).mul(e));
      if (Z(null == r ? void 0 : r.decimalNumber) && Z(null == r ? void 0 : r.fn)) return n;
      if (!Z(null == r ? void 0 : r.decimalNumber) && Z(null == r ? void 0 : r.fn)) return Number(n.toFixed(null == r ? void 0 : r.decimalNumber));

      if (!Z(null == r ? void 0 : r.decimalNumber) && !Z(null == r ? void 0 : r.fn)) {
        if ("toFixed" === (null == r ? void 0 : r.fn)) return Number(n.toFixed(null == r ? void 0 : r.decimalNumber));
        if ("floor" === (null == r ? void 0 : r.fn)) return Pt(n, null == r ? void 0 : r.decimalNumber);
      }
    },
          Rt = e => (t, r) => {
      const n = (null == r ? void 0 : r.decimalNumber) || 2,
            i = (null == r ? void 0 : r.fn) || "toFixed";
      if (We(t) || Z(t)) return t;
      if (isNaN(Number(t))) return NaN;
      const o = new $t(Number(t)).div(e);
      return "toFixed" === i ? Number(o.toFixed(n)) : "floor" === i ? Pt(Number(o), n) : t;
    },
          Ct = e => {
      const t = Object.prototype.toString.call(e),
            r = /^\[object (.*)\]$/.exec(t);
      return null !== r ? r[1].toLowerCase() : null;
    },
          qt = e => "string" == typeof e ? Ke(e) : ("array" !== Ct(e) && "object" !== Ct(e) || (Array.isArray(e) && e.forEach(e => {
      qt(e);
    }), null === typeof e) || Object.keys(e).map(t => {
      const r = e[t];
      "string" == typeof r && (e[t] = Ke(r)), "object" !== Ct(r) && "array" !== Ct(r) || qt(r);
    }), e);

    var Ht,
        Wt = new Uint8Array(16);

    function Yt() {
      if (!Ht && !(Ht = "undefined" != typeof crypto && crypto.getRandomValues && crypto.getRandomValues.bind(crypto) || "undefined" != typeof msCrypto && "function" == typeof msCrypto.getRandomValues && msCrypto.getRandomValues.bind(msCrypto))) throw new Error("crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported");
      return Ht(Wt);
    }

    var Vt = /^(?:[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}|00000000-0000-0000-0000-000000000000)$/i;

    function Qt(e) {
      return "string" == typeof e && Vt.test(e);
    }

    for (var Zt = [], Gt = 0; Gt < 256; ++Gt) Zt.push((Gt + 256).toString(16).substr(1));

    function Jt(e, t, r) {
      var n = (e = e || {}).random || (e.rng || Yt)();

      if (n[6] = 15 & n[6] | 64, n[8] = 63 & n[8] | 128, t) {
        r = r || 0;

        for (var i = 0; i < 16; ++i) t[r + i] = n[i];

        return t;
      }

      return function (e) {
        var t = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : 0,
            r = (Zt[e[t + 0]] + Zt[e[t + 1]] + Zt[e[t + 2]] + Zt[e[t + 3]] + "-" + Zt[e[t + 4]] + Zt[e[t + 5]] + "-" + Zt[e[t + 6]] + Zt[e[t + 7]] + "-" + Zt[e[t + 8]] + Zt[e[t + 9]] + "-" + Zt[e[t + 10]] + Zt[e[t + 11]] + Zt[e[t + 12]] + Zt[e[t + 13]] + Zt[e[t + 14]] + Zt[e[t + 15]]).toLowerCase();
        if (!Qt(r)) throw TypeError("Stringified UUID is invalid");
        return r;
      }(n);
    }

    const Xt = "input is invalid type",
          Kt = "0123456789abcdef".split(""),
          er = [128, 32768, 8388608, -2147483648];
    let tr,
        rr = [];
    const nr = new ArrayBuffer(68);
    tr = new Uint8Array(nr), rr = new Uint32Array(nr);

    function ir() {
      rr[0] = rr[16] = rr[1] = rr[2] = rr[3] = rr[4] = rr[5] = rr[6] = rr[7] = rr[8] = rr[9] = rr[10] = rr[11] = rr[12] = rr[13] = rr[14] = rr[15] = 0, this.blocks = rr, this.buffer8 = tr, this.h0 = this.h1 = this.h2 = this.h3 = this.start = this.bytes = this.hBytes = 0, this.finalized = this.hashed = !1, this.first = !0;
    }

    ir.prototype.update = function (e) {
      if (this.finalized) return;
      let t;
      const r = typeof e;

      if ("string" !== r) {
        if ("object" !== r) throw Xt;
        if (null === e) throw Xt;
        if (e.constructor === ArrayBuffer) e = new Uint8Array(e);else if (!Array.isArray(e) && !ArrayBuffer.isView(e)) throw Xt;
        t = !0;
      }

      let n,
          i,
          o = 0,
          s = e.length,
          a = this.blocks,
          c = this.buffer8;

      for (; o < s;) {
        if (this.hashed && (this.hashed = !1, a[0] = a[16], a[16] = a[1] = a[2] = a[3] = a[4] = a[5] = a[6] = a[7] = a[8] = a[9] = a[10] = a[11] = a[12] = a[13] = a[14] = a[15] = 0), t) for (i = this.start; o < s && i < 64; ++o) c[i++] = e[o];else for (i = this.start; o < s && i < 64; ++o) n = e.charCodeAt(o), n < 128 ? c[i++] = n : n < 2048 ? (c[i++] = 192 | n >> 6, c[i++] = 128 | 63 & n) : n < 55296 || n >= 57344 ? (c[i++] = 224 | n >> 12, c[i++] = 128 | n >> 6 & 63, c[i++] = 128 | 63 & n) : (n = 65536 + ((1023 & n) << 10 | 1023 & e.charCodeAt(++o)), c[i++] = 240 | n >> 18, c[i++] = 128 | n >> 12 & 63, c[i++] = 128 | n >> 6 & 63, c[i++] = 128 | 63 & n);
        this.lastByteIndex = i, this.bytes += i - this.start, i >= 64 ? (this.start = i - 64, this.hash(), this.hashed = !0) : this.start = i;
      }

      return this.bytes > 4294967295 && (this.hBytes += this.bytes / 4294967296 << 0, this.bytes = this.bytes % 4294967296), this;
    }, ir.prototype.finalize = function () {
      if (this.finalized) return;
      this.finalized = !0;
      const e = this.blocks,
            t = this.lastByteIndex;
      e[t >> 2] |= er[3 & t], t >= 56 && (this.hashed || this.hash(), e[0] = e[16], e[16] = e[1] = e[2] = e[3] = e[4] = e[5] = e[6] = e[7] = e[8] = e[9] = e[10] = e[11] = e[12] = e[13] = e[14] = e[15] = 0), e[14] = this.bytes << 3, e[15] = this.hBytes << 3 | this.bytes >>> 29, this.hash();
    }, ir.prototype.hash = function () {
      let e,
          t,
          r,
          n,
          i,
          o,
          s = this.blocks;
      this.first ? (e = s[0] - 680876937, e = (e << 7 | e >>> 25) - 271733879 << 0, n = (-1732584194 ^ 2004318071 & e) + s[1] - 117830708, n = (n << 12 | n >>> 20) + e << 0, r = (-271733879 ^ n & (-271733879 ^ e)) + s[2] - 1126478375, r = (r << 17 | r >>> 15) + n << 0, t = (e ^ r & (n ^ e)) + s[3] - 1316259209, t = (t << 22 | t >>> 10) + r << 0) : (e = this.h0, t = this.h1, r = this.h2, n = this.h3, e += (n ^ t & (r ^ n)) + s[0] - 680876936, e = (e << 7 | e >>> 25) + t << 0, n += (r ^ e & (t ^ r)) + s[1] - 389564586, n = (n << 12 | n >>> 20) + e << 0, r += (t ^ n & (e ^ t)) + s[2] + 606105819, r = (r << 17 | r >>> 15) + n << 0, t += (e ^ r & (n ^ e)) + s[3] - 1044525330, t = (t << 22 | t >>> 10) + r << 0), e += (n ^ t & (r ^ n)) + s[4] - 176418897, e = (e << 7 | e >>> 25) + t << 0, n += (r ^ e & (t ^ r)) + s[5] + 1200080426, n = (n << 12 | n >>> 20) + e << 0, r += (t ^ n & (e ^ t)) + s[6] - 1473231341, r = (r << 17 | r >>> 15) + n << 0, t += (e ^ r & (n ^ e)) + s[7] - 45705983, t = (t << 22 | t >>> 10) + r << 0, e += (n ^ t & (r ^ n)) + s[8] + 1770035416, e = (e << 7 | e >>> 25) + t << 0, n += (r ^ e & (t ^ r)) + s[9] - 1958414417, n = (n << 12 | n >>> 20) + e << 0, r += (t ^ n & (e ^ t)) + s[10] - 42063, r = (r << 17 | r >>> 15) + n << 0, t += (e ^ r & (n ^ e)) + s[11] - 1990404162, t = (t << 22 | t >>> 10) + r << 0, e += (n ^ t & (r ^ n)) + s[12] + 1804603682, e = (e << 7 | e >>> 25) + t << 0, n += (r ^ e & (t ^ r)) + s[13] - 40341101, n = (n << 12 | n >>> 20) + e << 0, r += (t ^ n & (e ^ t)) + s[14] - 1502002290, r = (r << 17 | r >>> 15) + n << 0, t += (e ^ r & (n ^ e)) + s[15] + 1236535329, t = (t << 22 | t >>> 10) + r << 0, e += (r ^ n & (t ^ r)) + s[1] - 165796510, e = (e << 5 | e >>> 27) + t << 0, n += (t ^ r & (e ^ t)) + s[6] - 1069501632, n = (n << 9 | n >>> 23) + e << 0, r += (e ^ t & (n ^ e)) + s[11] + 643717713, r = (r << 14 | r >>> 18) + n << 0, t += (n ^ e & (r ^ n)) + s[0] - 373897302, t = (t << 20 | t >>> 12) + r << 0, e += (r ^ n & (t ^ r)) + s[5] - 701558691, e = (e << 5 | e >>> 27) + t << 0, n += (t ^ r & (e ^ t)) + s[10] + 38016083, n = (n << 9 | n >>> 23) + e << 0, r += (e ^ t & (n ^ e)) + s[15] - 660478335, r = (r << 14 | r >>> 18) + n << 0, t += (n ^ e & (r ^ n)) + s[4] - 405537848, t = (t << 20 | t >>> 12) + r << 0, e += (r ^ n & (t ^ r)) + s[9] + 568446438, e = (e << 5 | e >>> 27) + t << 0, n += (t ^ r & (e ^ t)) + s[14] - 1019803690, n = (n << 9 | n >>> 23) + e << 0, r += (e ^ t & (n ^ e)) + s[3] - 187363961, r = (r << 14 | r >>> 18) + n << 0, t += (n ^ e & (r ^ n)) + s[8] + 1163531501, t = (t << 20 | t >>> 12) + r << 0, e += (r ^ n & (t ^ r)) + s[13] - 1444681467, e = (e << 5 | e >>> 27) + t << 0, n += (t ^ r & (e ^ t)) + s[2] - 51403784, n = (n << 9 | n >>> 23) + e << 0, r += (e ^ t & (n ^ e)) + s[7] + 1735328473, r = (r << 14 | r >>> 18) + n << 0, t += (n ^ e & (r ^ n)) + s[12] - 1926607734, t = (t << 20 | t >>> 12) + r << 0, i = t ^ r, e += (i ^ n) + s[5] - 378558, e = (e << 4 | e >>> 28) + t << 0, n += (i ^ e) + s[8] - 2022574463, n = (n << 11 | n >>> 21) + e << 0, o = n ^ e, r += (o ^ t) + s[11] + 1839030562, r = (r << 16 | r >>> 16) + n << 0, t += (o ^ r) + s[14] - 35309556, t = (t << 23 | t >>> 9) + r << 0, i = t ^ r, e += (i ^ n) + s[1] - 1530992060, e = (e << 4 | e >>> 28) + t << 0, n += (i ^ e) + s[4] + 1272893353, n = (n << 11 | n >>> 21) + e << 0, o = n ^ e, r += (o ^ t) + s[7] - 155497632, r = (r << 16 | r >>> 16) + n << 0, t += (o ^ r) + s[10] - 1094730640, t = (t << 23 | t >>> 9) + r << 0, i = t ^ r, e += (i ^ n) + s[13] + 681279174, e = (e << 4 | e >>> 28) + t << 0, n += (i ^ e) + s[0] - 358537222, n = (n << 11 | n >>> 21) + e << 0, o = n ^ e, r += (o ^ t) + s[3] - 722521979, r = (r << 16 | r >>> 16) + n << 0, t += (o ^ r) + s[6] + 76029189, t = (t << 23 | t >>> 9) + r << 0, i = t ^ r, e += (i ^ n) + s[9] - 640364487, e = (e << 4 | e >>> 28) + t << 0, n += (i ^ e) + s[12] - 421815835, n = (n << 11 | n >>> 21) + e << 0, o = n ^ e, r += (o ^ t) + s[15] + 530742520, r = (r << 16 | r >>> 16) + n << 0, t += (o ^ r) + s[2] - 995338651, t = (t << 23 | t >>> 9) + r << 0, e += (r ^ (t | ~n)) + s[0] - 198630844, e = (e << 6 | e >>> 26) + t << 0, n += (t ^ (e | ~r)) + s[7] + 1126891415, n = (n << 10 | n >>> 22) + e << 0, r += (e ^ (n | ~t)) + s[14] - 1416354905, r = (r << 15 | r >>> 17) + n << 0, t += (n ^ (r | ~e)) + s[5] - 57434055, t = (t << 21 | t >>> 11) + r << 0, e += (r ^ (t | ~n)) + s[12] + 1700485571, e = (e << 6 | e >>> 26) + t << 0, n += (t ^ (e | ~r)) + s[3] - 1894986606, n = (n << 10 | n >>> 22) + e << 0, r += (e ^ (n | ~t)) + s[10] - 1051523, r = (r << 15 | r >>> 17) + n << 0, t += (n ^ (r | ~e)) + s[1] - 2054922799, t = (t << 21 | t >>> 11) + r << 0, e += (r ^ (t | ~n)) + s[8] + 1873313359, e = (e << 6 | e >>> 26) + t << 0, n += (t ^ (e | ~r)) + s[15] - 30611744, n = (n << 10 | n >>> 22) + e << 0, r += (e ^ (n | ~t)) + s[6] - 1560198380, r = (r << 15 | r >>> 17) + n << 0, t += (n ^ (r | ~e)) + s[13] + 1309151649, t = (t << 21 | t >>> 11) + r << 0, e += (r ^ (t | ~n)) + s[4] - 145523070, e = (e << 6 | e >>> 26) + t << 0, n += (t ^ (e | ~r)) + s[11] - 1120210379, n = (n << 10 | n >>> 22) + e << 0, r += (e ^ (n | ~t)) + s[2] + 718787259, r = (r << 15 | r >>> 17) + n << 0, t += (n ^ (r | ~e)) + s[9] - 343485551, t = (t << 21 | t >>> 11) + r << 0, this.first ? (this.h0 = e + 1732584193 << 0, this.h1 = t - 271733879 << 0, this.h2 = r - 1732584194 << 0, this.h3 = n + 271733878 << 0, this.first = !1) : (this.h0 = this.h0 + e << 0, this.h1 = this.h1 + t << 0, this.h2 = this.h2 + r << 0, this.h3 = this.h3 + n << 0);
    }, ir.prototype.hex = function () {
      this.finalize();
      let e = this.h0,
          t = this.h1,
          r = this.h2,
          n = this.h3;
      return Kt[e >> 4 & 15] + Kt[15 & e] + Kt[e >> 12 & 15] + Kt[e >> 8 & 15] + Kt[e >> 20 & 15] + Kt[e >> 16 & 15] + Kt[e >> 28 & 15] + Kt[e >> 24 & 15] + Kt[t >> 4 & 15] + Kt[15 & t] + Kt[t >> 12 & 15] + Kt[t >> 8 & 15] + Kt[t >> 20 & 15] + Kt[t >> 16 & 15] + Kt[t >> 28 & 15] + Kt[t >> 24 & 15] + Kt[r >> 4 & 15] + Kt[15 & r] + Kt[r >> 12 & 15] + Kt[r >> 8 & 15] + Kt[r >> 20 & 15] + Kt[r >> 16 & 15] + Kt[r >> 28 & 15] + Kt[r >> 24 & 15] + Kt[n >> 4 & 15] + Kt[15 & n] + Kt[n >> 12 & 15] + Kt[n >> 8 & 15] + Kt[n >> 20 & 15] + Kt[n >> 16 & 15] + Kt[n >> 28 & 15] + Kt[n >> 24 & 15];
    };

    var or = e => new ir().update(e).hex();

    function sr(e) {
      const t = typeof e;
      return null != e && ("object" === t || "function" === t);
    }

    function ar(e, t, r) {
      if ("undefined" == typeof window) return;
      let n,
          i,
          o,
          s,
          a,
          c,
          u = 0,
          f = !1,
          l = !1,
          h = !0;
      const m = !t && 0 !== t && "function" == typeof requestAnimationFrame;
      if ("function" != typeof e) throw new TypeError("Expected a function");

      function d(t) {
        const r = n,
              o = i;
        return n = i = void 0, u = t, s = e.apply(o, r), s;
      }

      function p(e, t) {
        return m ? (cancelAnimationFrame(a), requestAnimationFrame(e)) : setTimeout(e, t);
      }

      function g(e) {
        const r = e - c;
        return void 0 === c || r >= t || r < 0 || l && e - u >= o;
      }

      function y() {
        const e = Date.now();
        if (g(e)) return w(e);
        a = p(y, function (e) {
          const r = e - u,
                n = t - (e - c);
          return l ? Math.min(n, o - r) : n;
        }(e));
      }

      function w(e) {
        return a = void 0, h && n ? d(e) : (n = i = void 0, s);
      }

      function x(...e) {
        const r = Date.now(),
              o = g(r);

        if (n = e, i = this, c = r, o) {
          if (void 0 === a) return function (e) {
            return u = e, a = p(y, t), f ? d(e) : s;
          }(c);
          if (l) return a = p(y, t), d(c);
        }

        return void 0 === a && (a = p(y, t)), s;
      }

      return t = +t || 0, sr(r) && (f = !!r.leading, l = "maxWait" in r, o = l ? Math.max(+r.maxWait || 0, t) : o, h = "trailing" in r ? !!r.trailing : h), x.cancel = function () {
        void 0 !== a && function (e) {
          if (m) return cancelAnimationFrame(e);
          clearTimeout(e);
        }(a), u = 0, n = c = i = a = void 0;
      }, x.flush = function () {
        return void 0 === a ? s : w(Date.now());
      }, x.pending = function () {
        return void 0 !== a;
      }, x;
    }

    var cr = {},
        ur = {
      /*! ieee754. BSD-3-Clause License. Feross Aboukhadijeh <https://feross.org/opensource> */
      read: function (e, t, r, n, i) {
        var o,
            s,
            a = 8 * i - n - 1,
            c = (1 << a) - 1,
            u = c >> 1,
            f = -7,
            l = r ? i - 1 : 0,
            h = r ? -1 : 1,
            m = e[t + l];

        for (l += h, o = m & (1 << -f) - 1, m >>= -f, f += a; f > 0; o = 256 * o + e[t + l], l += h, f -= 8);

        for (s = o & (1 << -f) - 1, o >>= -f, f += n; f > 0; s = 256 * s + e[t + l], l += h, f -= 8);

        if (0 === o) o = 1 - u;else {
          if (o === c) return s ? NaN : 1 / 0 * (m ? -1 : 1);
          s += Math.pow(2, n), o -= u;
        }
        return (m ? -1 : 1) * s * Math.pow(2, o - n);
      },
      write: function (e, t, r, n, i, o) {
        var s,
            a,
            c,
            u = 8 * o - i - 1,
            f = (1 << u) - 1,
            l = f >> 1,
            h = 23 === i ? Math.pow(2, -24) - Math.pow(2, -77) : 0,
            m = n ? 0 : o - 1,
            d = n ? 1 : -1,
            p = t < 0 || 0 === t && 1 / t < 0 ? 1 : 0;

        for (t = Math.abs(t), isNaN(t) || t === 1 / 0 ? (a = isNaN(t) ? 1 : 0, s = f) : (s = Math.floor(Math.log(t) / Math.LN2), t * (c = Math.pow(2, -s)) < 1 && (s--, c *= 2), (t += s + l >= 1 ? h / c : h * Math.pow(2, 1 - l)) * c >= 2 && (s++, c /= 2), s + l >= f ? (a = 0, s = f) : s + l >= 1 ? (a = (t * c - 1) * Math.pow(2, i), s += l) : (a = t * Math.pow(2, l - 1) * Math.pow(2, i), s = 0)); i >= 8; e[r + m] = 255 & a, m += d, a /= 256, i -= 8);

        for (s = s << i | a, u += i; u > 0; e[r + m] = 255 & s, m += d, s /= 256, u -= 8);

        e[r + m - d] |= 128 * p;
      }
    };
    !function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.AnsiStringType = e.StringType = e.BufferType = e.Uint8ArrayType = e.IgnoreType = e.Float80_LE = e.Float80_BE = e.Float64_LE = e.Float64_BE = e.Float32_LE = e.Float32_BE = e.Float16_LE = e.Float16_BE = e.INT64_BE = e.UINT64_BE = e.INT64_LE = e.UINT64_LE = e.INT32_LE = e.INT32_BE = e.INT24_BE = e.INT24_LE = e.INT16_LE = e.INT16_BE = e.INT8 = e.UINT32_BE = e.UINT32_LE = e.UINT24_BE = e.UINT24_LE = e.UINT16_BE = e.UINT16_LE = e.UINT8 = void 0;
      const t = ur;

      function r(e) {
        return new DataView(e.buffer, e.byteOffset);
      }

      e.UINT8 = {
        len: 1,
        get: (e, t) => r(e).getUint8(t),
        put: (e, t, n) => (r(e).setUint8(t, n), t + 1)
      }, e.UINT16_LE = {
        len: 2,
        get: (e, t) => r(e).getUint16(t, !0),
        put: (e, t, n) => (r(e).setUint16(t, n, !0), t + 2)
      }, e.UINT16_BE = {
        len: 2,
        get: (e, t) => r(e).getUint16(t),
        put: (e, t, n) => (r(e).setUint16(t, n), t + 2)
      }, e.UINT24_LE = {
        len: 3,

        get(e, t) {
          const n = r(e);
          return n.getUint8(t) + (n.getUint16(t + 1, !0) << 8);
        },

        put(e, t, n) {
          const i = r(e);
          return i.setUint8(t, 255 & n), i.setUint16(t + 1, n >> 8, !0), t + 3;
        }

      }, e.UINT24_BE = {
        len: 3,

        get(e, t) {
          const n = r(e);
          return (n.getUint16(t) << 8) + n.getUint8(t + 2);
        },

        put(e, t, n) {
          const i = r(e);
          return i.setUint16(t, n >> 8), i.setUint8(t + 2, 255 & n), t + 3;
        }

      }, e.UINT32_LE = {
        len: 4,
        get: (e, t) => r(e).getUint32(t, !0),
        put: (e, t, n) => (r(e).setUint32(t, n, !0), t + 4)
      }, e.UINT32_BE = {
        len: 4,
        get: (e, t) => r(e).getUint32(t),
        put: (e, t, n) => (r(e).setUint32(t, n), t + 4)
      }, e.INT8 = {
        len: 1,
        get: (e, t) => r(e).getInt8(t),
        put: (e, t, n) => (r(e).setInt8(t, n), t + 1)
      }, e.INT16_BE = {
        len: 2,
        get: (e, t) => r(e).getInt16(t),
        put: (e, t, n) => (r(e).setInt16(t, n), t + 2)
      }, e.INT16_LE = {
        len: 2,
        get: (e, t) => r(e).getInt16(t, !0),
        put: (e, t, n) => (r(e).setInt16(t, n, !0), t + 2)
      }, e.INT24_LE = {
        len: 3,

        get(t, r) {
          const n = e.UINT24_LE.get(t, r);
          return n > 8388607 ? n - 16777216 : n;
        },

        put(e, t, n) {
          const i = r(e);
          return i.setUint8(t, 255 & n), i.setUint16(t + 1, n >> 8, !0), t + 3;
        }

      }, e.INT24_BE = {
        len: 3,

        get(t, r) {
          const n = e.UINT24_BE.get(t, r);
          return n > 8388607 ? n - 16777216 : n;
        },

        put(e, t, n) {
          const i = r(e);
          return i.setUint16(t, n >> 8), i.setUint8(t + 2, 255 & n), t + 3;
        }

      }, e.INT32_BE = {
        len: 4,
        get: (e, t) => r(e).getInt32(t),
        put: (e, t, n) => (r(e).setInt32(t, n), t + 4)
      }, e.INT32_LE = {
        len: 4,
        get: (e, t) => r(e).getInt32(t, !0),
        put: (e, t, n) => (r(e).setInt32(t, n, !0), t + 4)
      }, e.UINT64_LE = {
        len: 8,
        get: (e, t) => r(e).getBigUint64(t, !0),
        put: (e, t, n) => (r(e).setBigUint64(t, n, !0), t + 8)
      }, e.INT64_LE = {
        len: 8,
        get: (e, t) => r(e).getBigInt64(t, !0),
        put: (e, t, n) => (r(e).setBigInt64(t, n, !0), t + 8)
      }, e.UINT64_BE = {
        len: 8,
        get: (e, t) => r(e).getBigUint64(t),
        put: (e, t, n) => (r(e).setBigUint64(t, n), t + 8)
      }, e.INT64_BE = {
        len: 8,
        get: (e, t) => r(e).getBigInt64(t),
        put: (e, t, n) => (r(e).setBigInt64(t, n), t + 8)
      }, e.Float16_BE = {
        len: 2,

        get(e, r) {
          return t.read(e, r, !1, 10, this.len);
        },

        put(e, r, n) {
          return t.write(e, n, r, !1, 10, this.len), r + this.len;
        }

      }, e.Float16_LE = {
        len: 2,

        get(e, r) {
          return t.read(e, r, !0, 10, this.len);
        },

        put(e, r, n) {
          return t.write(e, n, r, !0, 10, this.len), r + this.len;
        }

      }, e.Float32_BE = {
        len: 4,
        get: (e, t) => r(e).getFloat32(t),
        put: (e, t, n) => (r(e).setFloat32(t, n), t + 4)
      }, e.Float32_LE = {
        len: 4,
        get: (e, t) => r(e).getFloat32(t, !0),
        put: (e, t, n) => (r(e).setFloat32(t, n, !0), t + 4)
      }, e.Float64_BE = {
        len: 8,
        get: (e, t) => r(e).getFloat64(t),
        put: (e, t, n) => (r(e).setFloat64(t, n), t + 8)
      }, e.Float64_LE = {
        len: 8,
        get: (e, t) => r(e).getFloat64(t, !0),
        put: (e, t, n) => (r(e).setFloat64(t, n, !0), t + 8)
      }, e.Float80_BE = {
        len: 10,

        get(e, r) {
          return t.read(e, r, !1, 63, this.len);
        },

        put(e, r, n) {
          return t.write(e, n, r, !1, 63, this.len), r + this.len;
        }

      }, e.Float80_LE = {
        len: 10,

        get(e, r) {
          return t.read(e, r, !0, 63, this.len);
        },

        put(e, r, n) {
          return t.write(e, n, r, !0, 63, this.len), r + this.len;
        }

      };
      e.IgnoreType = class {
        constructor(e) {
          this.len = e;
        }

        get(e, t) {}

      };
      e.Uint8ArrayType = class {
        constructor(e) {
          this.len = e;
        }

        get(e, t) {
          return e.subarray(t, t + this.len);
        }

      };
      e.BufferType = class {
        constructor(e) {
          this.len = e;
        }

        get(e, t) {
          return Buffer.from(e.subarray(t, t + this.len));
        }

      };
      e.StringType = class {
        constructor(e, t) {
          this.len = e, this.encoding = t;
        }

        get(e, t) {
          return Buffer.from(e).toString(this.encoding, t, t + this.len);
        }

      };

      class n {
        constructor(e) {
          this.len = e;
        }

        static decode(e, t, r) {
          let i = "";

          for (let o = t; o < r; ++o) i += n.codePointToString(n.singleByteDecoder(e[o]));

          return i;
        }

        static inRange(e, t, r) {
          return t <= e && e <= r;
        }

        static codePointToString(e) {
          return e <= 65535 ? String.fromCharCode(e) : (e -= 65536, String.fromCharCode(55296 + (e >> 10), 56320 + (1023 & e)));
        }

        static singleByteDecoder(e) {
          if (n.inRange(e, 0, 127)) return e;
          const t = n.windows1252[e - 128];
          if (null === t) throw Error("invaliding encoding");
          return t;
        }

        get(e, t = 0) {
          return n.decode(e, t, t + this.len);
        }

      }

      e.AnsiStringType = n, n.windows1252 = [8364, 129, 8218, 402, 8222, 8230, 8224, 8225, 710, 8240, 352, 8249, 338, 141, 381, 143, 144, 8216, 8217, 8220, 8221, 8226, 8211, 8212, 732, 8482, 353, 8250, 339, 157, 382, 376, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255];
    }(cr);
    var fr = {},
        lr = {};
    Object.defineProperty(lr, "__esModule", {
      value: !0
    }), lr.readFile = lr.writeFileSync = lr.writeFile = lr.read = lr.open = lr.close = lr.stat = lr.createReadStream = lr.pathExists = void 0;
    const hr = n.default;
    lr.pathExists = hr.existsSync, lr.createReadStream = hr.createReadStream, lr.stat = async function (e) {
      return new Promise((t, r) => {
        hr.stat(e, (e, n) => {
          e ? r(e) : t(n);
        });
      });
    }, lr.close = async function (e) {
      return new Promise((t, r) => {
        hr.close(e, e => {
          e ? r(e) : t();
        });
      });
    }, lr.open = async function (e, t) {
      return new Promise((r, n) => {
        hr.open(e, t, (e, t) => {
          e ? n(e) : r(t);
        });
      });
    }, lr.read = async function (e, t, r, n, i) {
      return new Promise((o, s) => {
        hr.read(e, t, r, n, i, (e, t, r) => {
          e ? s(e) : o({
            bytesRead: t,
            buffer: r
          });
        });
      });
    }, lr.writeFile = async function (e, t) {
      return new Promise((r, n) => {
        hr.writeFile(e, t, e => {
          e ? n(e) : r();
        });
      });
    }, lr.writeFileSync = function (e, t) {
      hr.writeFileSync(e, t);
    }, lr.readFile = async function (e) {
      return new Promise((t, r) => {
        hr.readFile(e, (e, n) => {
          e ? r(e) : t(n);
        });
      });
    };
    var mr = {},
        dr = {},
        pr = {},
        gr = {},
        yr = {};
    !function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.EndOfStreamError = e.defaultMessages = void 0, e.defaultMessages = "End-Of-Stream";

      class t extends Error {
        constructor() {
          super(e.defaultMessages);
        }

      }

      e.EndOfStreamError = t;
    }(yr);
    var wr = {},
        xr = {};
    Object.defineProperty(xr, "__esModule", {
      value: !0
    }), xr.Deferred = void 0;
    xr.Deferred = class {
      constructor() {
        this.resolve = () => null, this.reject = () => null, this.promise = new Promise((e, t) => {
          this.reject = t, this.resolve = e;
        });
      }

    }, function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.StreamReader = e.EndOfStreamError = void 0;
      const t = yr,
            r = xr;
      var n = yr;
      Object.defineProperty(e, "EndOfStreamError", {
        enumerable: !0,
        get: function () {
          return n.EndOfStreamError;
        }
      });
      e.StreamReader = class {
        constructor(e) {
          if (this.s = e, this.deferred = null, this.endOfStream = !1, this.peekQueue = [], !e.read || !e.once) throw new Error("Expected an instance of stream.Readable");
          this.s.once("end", () => this.reject(new t.EndOfStreamError())), this.s.once("error", e => this.reject(e)), this.s.once("close", () => this.reject(new Error("Stream closed")));
        }

        async peek(e, t, r) {
          const n = await this.read(e, t, r);
          return this.peekQueue.push(e.subarray(t, t + n)), n;
        }

        async read(e, r, n) {
          if (0 === n) return 0;
          if (0 === this.peekQueue.length && this.endOfStream) throw new t.EndOfStreamError();
          let i = n,
              o = 0;

          for (; this.peekQueue.length > 0 && i > 0;) {
            const t = this.peekQueue.pop();
            if (!t) throw new Error("peekData should be defined");
            const n = Math.min(t.length, i);
            e.set(t.subarray(0, n), r + o), o += n, i -= n, n < t.length && this.peekQueue.push(t.subarray(n));
          }

          for (; i > 0 && !this.endOfStream;) {
            const t = Math.min(i, 1048576),
                  n = await this.readFromStream(e, r + o, t);
            if (o += n, n < t) break;
            i -= n;
          }

          return o;
        }

        async readFromStream(e, t, n) {
          const i = this.s.read(n);
          if (i) return e.set(i, t), i.length;
          {
            const i = {
              buffer: e,
              offset: t,
              length: n,
              deferred: new r.Deferred()
            };
            return this.deferred = i.deferred, this.s.once("readable", () => {
              this.readDeferred(i);
            }), i.deferred.promise;
          }
        }

        readDeferred(e) {
          const t = this.s.read(e.length);
          t ? (e.buffer.set(t, e.offset), e.deferred.resolve(t.length), this.deferred = null) : this.s.once("readable", () => {
            this.readDeferred(e);
          });
        }

        reject(e) {
          this.endOfStream = !0, this.deferred && (this.deferred.reject(e), this.deferred = null);
        }

      };
    }(wr), function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.StreamReader = e.EndOfStreamError = void 0;
      var t = yr;
      Object.defineProperty(e, "EndOfStreamError", {
        enumerable: !0,
        get: function () {
          return t.EndOfStreamError;
        }
      });
      var r = wr;
      Object.defineProperty(e, "StreamReader", {
        enumerable: !0,
        get: function () {
          return r.StreamReader;
        }
      });
    }(gr), Object.defineProperty(pr, "__esModule", {
      value: !0
    }), pr.AbstractTokenizer = void 0;
    const br = gr;
    pr.AbstractTokenizer = class {
      constructor(e) {
        this.position = 0, this.numBuffer = new Uint8Array(8), this.fileInfo = e || {};
      }

      async readToken(e, t = this.position) {
        const r = Buffer.alloc(e.len);
        if ((await this.readBuffer(r, {
          position: t
        })) < e.len) throw new br.EndOfStreamError();
        return e.get(r, 0);
      }

      async peekToken(e, t = this.position) {
        const r = Buffer.alloc(e.len);
        if ((await this.peekBuffer(r, {
          position: t
        })) < e.len) throw new br.EndOfStreamError();
        return e.get(r, 0);
      }

      async readNumber(e) {
        if ((await this.readBuffer(this.numBuffer, {
          length: e.len
        })) < e.len) throw new br.EndOfStreamError();
        return e.get(this.numBuffer, 0);
      }

      async peekNumber(e) {
        if ((await this.peekBuffer(this.numBuffer, {
          length: e.len
        })) < e.len) throw new br.EndOfStreamError();
        return e.get(this.numBuffer, 0);
      }

      async ignore(e) {
        if (void 0 !== this.fileInfo.size) {
          const t = this.fileInfo.size - this.position;
          if (e > t) return this.position += t, t;
        }

        return this.position += e, e;
      }

      async close() {}

      normalizeOptions(e, t) {
        if (t && void 0 !== t.position && t.position < this.position) throw new Error("`options.position` must be equal or greater than `tokenizer.position`");
        return t ? {
          mayBeLess: !0 === t.mayBeLess,
          offset: t.offset ? t.offset : 0,
          length: t.length ? t.length : e.length - (t.offset ? t.offset : 0),
          position: t.position ? t.position : this.position
        } : {
          mayBeLess: !1,
          offset: 0,
          length: e.length,
          position: this.position
        };
      }

    }, Object.defineProperty(dr, "__esModule", {
      value: !0
    }), dr.ReadStreamTokenizer = void 0;
    const vr = pr,
          kr = gr;

    class Sr extends vr.AbstractTokenizer {
      constructor(e, t) {
        super(t), this.streamReader = new kr.StreamReader(e);
      }

      async getFileInfo() {
        return this.fileInfo;
      }

      async readBuffer(e, t) {
        const r = this.normalizeOptions(e, t),
              n = r.position - this.position;
        if (n > 0) return await this.ignore(n), this.readBuffer(e, t);
        if (n < 0) throw new Error("`options.position` must be equal or greater than `tokenizer.position`");
        if (0 === r.length) return 0;
        const i = await this.streamReader.read(e, r.offset, r.length);
        if (this.position += i, (!t || !t.mayBeLess) && i < r.length) throw new kr.EndOfStreamError();
        return i;
      }

      async peekBuffer(e, t) {
        const r = this.normalizeOptions(e, t);
        let n = 0;

        if (r.position) {
          const t = r.position - this.position;

          if (t > 0) {
            const i = new Uint8Array(r.length + t);
            return n = await this.peekBuffer(i, {
              mayBeLess: r.mayBeLess
            }), e.set(i.subarray(t), r.offset), n - t;
          }

          if (t < 0) throw new Error("Cannot peek from a negative offset in a stream");
        }

        if (r.length > 0) {
          try {
            n = await this.streamReader.peek(e, r.offset, r.length);
          } catch (e) {
            if (t && t.mayBeLess && e instanceof kr.EndOfStreamError) return 0;
            throw e;
          }

          if (!r.mayBeLess && n < r.length) throw new kr.EndOfStreamError();
        }

        return n;
      }

      async ignore(e) {
        const t = Math.min(256e3, e),
              r = new Uint8Array(t);
        let n = 0;

        for (; n < e;) {
          const i = e - n,
                o = await this.readBuffer(r, {
            length: Math.min(t, i)
          });
          if (o < 0) return o;
          n += o;
        }

        return n;
      }

    }

    dr.ReadStreamTokenizer = Sr;
    var Er = {};
    Object.defineProperty(Er, "__esModule", {
      value: !0
    }), Er.BufferTokenizer = void 0;
    const Tr = gr,
          Or = pr;

    class Nr extends Or.AbstractTokenizer {
      constructor(e, t) {
        super(t), this.uint8Array = e, this.fileInfo.size = this.fileInfo.size ? this.fileInfo.size : e.length;
      }

      async readBuffer(e, t) {
        if (t && t.position) {
          if (t.position < this.position) throw new Error("`options.position` must be equal or greater than `tokenizer.position`");
          this.position = t.position;
        }

        const r = await this.peekBuffer(e, t);
        return this.position += r, r;
      }

      async peekBuffer(e, t) {
        const r = this.normalizeOptions(e, t),
              n = Math.min(this.uint8Array.length - r.position, r.length);
        if (!r.mayBeLess && n < r.length) throw new Tr.EndOfStreamError();
        return e.set(this.uint8Array.subarray(r.position, r.position + n), r.offset), n;
      }

      async close() {}

    }

    Er.BufferTokenizer = Nr, function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.fromBuffer = e.fromStream = e.EndOfStreamError = void 0;
      const t = dr,
            r = Er;
      var n = gr;
      Object.defineProperty(e, "EndOfStreamError", {
        enumerable: !0,
        get: function () {
          return n.EndOfStreamError;
        }
      }), e.fromStream = function (e, r) {
        return r = r || {}, new t.ReadStreamTokenizer(e, r);
      }, e.fromBuffer = function (e, t) {
        return new r.BufferTokenizer(e, t);
      };
    }(mr);
    var Ir = {};
    Object.defineProperty(Ir, "__esModule", {
      value: !0
    }), Ir.fromFile = Ir.FileTokenizer = void 0;
    const jr = pr,
          _r = gr,
          Br = lr;

    class Mr extends jr.AbstractTokenizer {
      constructor(e, t) {
        super(t), this.fd = e;
      }

      async readBuffer(e, t) {
        const r = this.normalizeOptions(e, t);
        this.position = r.position;
        const n = await Br.read(this.fd, e, r.offset, r.length, r.position);
        if (this.position += n.bytesRead, n.bytesRead < r.length && (!t || !t.mayBeLess)) throw new _r.EndOfStreamError();
        return n.bytesRead;
      }

      async peekBuffer(e, t) {
        const r = this.normalizeOptions(e, t),
              n = await Br.read(this.fd, e, r.offset, r.length, r.position);
        if (!r.mayBeLess && n.bytesRead < r.length) throw new _r.EndOfStreamError();
        return n.bytesRead;
      }

      async close() {
        return Br.close(this.fd);
      }

    }

    Ir.FileTokenizer = Mr, Ir.fromFile = async function (e) {
      const t = await Br.stat(e);
      if (!t.isFile) throw new Error(`File not a file: ${e}`);
      const r = await Br.open(e, "r");
      return new Mr(r, {
        path: e,
        size: t.size
      });
    }, function (e) {
      Object.defineProperty(e, "__esModule", {
        value: !0
      }), e.fromStream = e.fromBuffer = e.EndOfStreamError = e.fromFile = void 0;
      const t = lr,
            r = mr;
      var n = Ir;
      Object.defineProperty(e, "fromFile", {
        enumerable: !0,
        get: function () {
          return n.fromFile;
        }
      });
      var i = mr;
      Object.defineProperty(e, "EndOfStreamError", {
        enumerable: !0,
        get: function () {
          return i.EndOfStreamError;
        }
      }), Object.defineProperty(e, "fromBuffer", {
        enumerable: !0,
        get: function () {
          return i.fromBuffer;
        }
      }), e.fromStream = async function (e, n) {
        if (n = n || {}, e.path) {
          const r = await t.stat(e.path);
          n.path = e.path, n.size = r.size;
        }

        return r.fromStream(e, n);
      };
    }(fr);
    const Ar = {
      get: (e, t) => 127 & e[t + 3] | e[t + 2] << 7 | e[t + 1] << 14 | e[t] << 21,
      len: 4
    };

    async function Fr(e) {
      try {
        return new Ur().parse(e);
      } catch (e) {
        if (!(e instanceof fr.EndOfStreamError)) throw e;
      }
    }

    function zr(e, t, r) {
      r = Object.assign({
        offset: 0
      }, r);

      for (const [n, i] of t.entries()) if (r.mask) {
        if (i !== (r.mask[n] & e[n + r.offset])) return !1;
      } else if (i !== e[n + r.offset]) return !1;

      return !0;
    }

    class Ur {
      constructor() {
        this.buffer = Buffer.alloc(4100);
      }

      check(e, t) {
        return zr(this.buffer, e, t);
      }

      checkString(e, t) {
        return this.check((r = e, Object.values(r).map(e => e.charCodeAt(0))), t);
        var r;
      }

      async readTiffTag(e) {
        if (this.tokenizer) {
          const t = await this.tokenizer.readToken(e ? cr.UINT16_BE : cr.UINT16_LE);

          switch (this.tokenizer.ignore(10), t) {
            case 50341:
              return {
                ext: "arw",
                mime: "image/x-sony-arw"
              };

            case 50706:
              return {
                ext: "dng",
                mime: "image/x-adobe-dng"
              };
          }
        }
      }

      async readTiffIFD(e) {
        if (this.tokenizer) {
          const t = await this.tokenizer.readToken(e ? cr.UINT16_BE : cr.UINT16_LE);

          for (let r = 0; r < t; ++r) {
            const t = await this.readTiffTag(e);
            if (t) return t;
          }
        }
      }

      async readTiffHeader(e) {
        const t = (e ? cr.UINT16_BE : cr.UINT16_LE).get(this.buffer, 2),
              r = (e ? cr.UINT32_BE : cr.UINT32_LE).get(this.buffer, 4);

        if (42 === t) {
          if (r >= 6) {
            if (this.checkString("CR", {
              offset: 8
            })) return {
              ext: "cr2",
              mime: "image/x-canon-cr2"
            };
            if (r >= 8 && (this.check([28, 0, 254, 0], {
              offset: 8
            }) || this.check([31, 0, 11, 0], {
              offset: 8
            }))) return {
              ext: "nef",
              mime: "image/x-nikon-nef"
            };
          }

          if (this.tokenizer) {
            await this.tokenizer.ignore(r);
            const e = await this.readTiffIFD(!1);
            return e || {
              ext: "tif",
              mime: "image/tiff"
            };
          }
        }

        if (43 === t) return {
          ext: "tif",
          mime: "image/tiff"
        };
      }

      async parse(e) {
        if (this.buffer = Buffer.alloc(4100), void 0 === e.fileInfo.size && (e.fileInfo.size = Number.MAX_SAFE_INTEGER), this.tokenizer = e, await e.peekBuffer(this.buffer, {
          length: 12,
          mayBeLess: !0
        }), this.check([66, 77])) return {
          ext: "bmp",
          mime: "image/bmp"
        };
        if (this.check([11, 119])) return {
          ext: "ac3",
          mime: "audio/vnd.dolby.dd-raw"
        };
        if (this.check([120, 1])) return {
          ext: "dmg",
          mime: "application/x-apple-diskimage"
        };
        if (this.check([77, 90])) return {
          ext: "exe",
          mime: "application/x-msdownload"
        };
        if (this.check([37, 33])) return await e.peekBuffer(this.buffer, {
          length: 24,
          mayBeLess: !0
        }), this.checkString("PS-Adobe-", {
          offset: 2
        }) && this.checkString(" EPSF-", {
          offset: 14
        }) ? {
          ext: "eps",
          mime: "application/eps"
        } : {
          ext: "ps",
          mime: "application/postscript"
        };
        if (this.check([31, 160]) || this.check([31, 157])) return {
          ext: "Z",
          mime: "application/x-compress"
        };
        if (this.check([71, 73, 70])) return {
          ext: "gif",
          mime: "image/gif"
        };
        if (this.check([255, 216, 255])) return {
          ext: "jpg",
          mime: "image/jpeg"
        };
        if (this.check([73, 73, 188])) return {
          ext: "jxr",
          mime: "image/vnd.ms-photo"
        };
        if (this.check([31, 139, 8])) return {
          ext: "gz",
          mime: "application/gzip"
        };
        if (this.check([66, 90, 104])) return {
          ext: "bz2",
          mime: "application/x-bzip2"
        };

        if (this.checkString("ID3")) {
          await e.ignore(6);
          const t = await e.readToken(Ar);
          return e.position + t > e.fileInfo.size ? {
            ext: "mp3",
            mime: "audio/mpeg"
          } : (await e.ignore(t), Fr(e));
        }

        if (this.checkString("MP+")) return {
          ext: "mpc",
          mime: "audio/x-musepack"
        };
        if ((67 === this.buffer[0] || 70 === this.buffer[0]) && this.check([87, 83], {
          offset: 1
        })) return {
          ext: "swf",
          mime: "application/x-shockwave-flash"
        };
        if (this.checkString("FLIF")) return {
          ext: "flif",
          mime: "image/flif"
        };
        if (this.checkString("8BPS")) return {
          ext: "psd",
          mime: "image/vnd.adobe.photoshop"
        };
        if (this.checkString("WEBP", {
          offset: 8
        })) return {
          ext: "webp",
          mime: "image/webp"
        };
        if (this.checkString("MPCK")) return {
          ext: "mpc",
          mime: "audio/x-musepack"
        };
        if (this.checkString("FORM")) return {
          ext: "aif",
          mime: "audio/aiff"
        };
        if (this.checkString("icns", {
          offset: 0
        })) return {
          ext: "icns",
          mime: "image/icns"
        };

        if (this.check([80, 75, 3, 4])) {
          try {
            for (; e.position + 30 < e.fileInfo.size;) {
              await e.readBuffer(this.buffer, {
                length: 30
              });
              const t = {
                compressedSize: this.buffer.readUInt32LE(18),
                uncompressedSize: this.buffer.readUInt32LE(22),
                filenameLength: this.buffer.readUInt16LE(26),
                extraFieldLength: this.buffer.readUInt16LE(28)
              };
              if (t.filename = await e.readToken(new cr.StringType(t.filenameLength, "utf-8")), await e.ignore(t.extraFieldLength), "META-INF/mozilla.rsa" === t.filename) return {
                ext: "xpi",
                mime: "application/x-xpinstall"
              };

              if (t.filename.endsWith(".rels") || t.filename.endsWith(".xml")) {
                switch (t.filename.split("/")[0]) {
                  case "_rels":
                  default:
                    break;

                  case "word":
                    return {
                      ext: "docx",
                      mime: "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
                    };

                  case "ppt":
                    return {
                      ext: "pptx",
                      mime: "application/vnd.openxmlformats-officedocument.presentationml.presentation"
                    };

                  case "xl":
                    return {
                      ext: "xlsx",
                      mime: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
                    };
                }
              }

              if (t.filename.startsWith("xl/")) return {
                ext: "xlsx",
                mime: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
              };
              if (t.filename.startsWith("3D/") && t.filename.endsWith(".model")) return {
                ext: "3mf",
                mime: "model/3mf"
              };

              if ("mimetype" === t.filename && t.compressedSize === t.uncompressedSize) {
                switch (await e.readToken(new cr.StringType(t.compressedSize, "utf-8"))) {
                  case "application/epub+zip":
                    return {
                      ext: "epub",
                      mime: "application/epub+zip"
                    };

                  case "application/vnd.oasis.opendocument.text":
                    return {
                      ext: "odt",
                      mime: "application/vnd.oasis.opendocument.text"
                    };

                  case "application/vnd.oasis.opendocument.spreadsheet":
                    return {
                      ext: "ods",
                      mime: "application/vnd.oasis.opendocument.spreadsheet"
                    };

                  case "application/vnd.oasis.opendocument.presentation":
                    return {
                      ext: "odp",
                      mime: "application/vnd.oasis.opendocument.presentation"
                    };
                }
              }

              if (0 === t.compressedSize) {
                let t = -1;

                for (; t < 0 && e.position < e.fileInfo.size;) await e.peekBuffer(this.buffer, {
                  mayBeLess: !0
                }), t = this.buffer.indexOf("504B0304", 0, "hex"), await e.ignore(t >= 0 ? t : this.buffer.length);
              } else await e.ignore(t.compressedSize);
            }
          } catch (e) {
            if (!(e instanceof fr.EndOfStreamError)) throw e;
          }

          return {
            ext: "zip",
            mime: "application/zip"
          };
        }

        if (this.checkString("OggS")) {
          await e.ignore(28);
          const t = Buffer.alloc(8);
          return await e.readBuffer(t), zr(t, [79, 112, 117, 115, 72, 101, 97, 100]) ? {
            ext: "opus",
            mime: "audio/opus"
          } : zr(t, [128, 116, 104, 101, 111, 114, 97]) ? {
            ext: "ogv",
            mime: "video/ogg"
          } : zr(t, [1, 118, 105, 100, 101, 111, 0]) ? {
            ext: "ogm",
            mime: "video/ogg"
          } : zr(t, [127, 70, 76, 65, 67]) ? {
            ext: "oga",
            mime: "audio/ogg"
          } : zr(t, [83, 112, 101, 101, 120, 32, 32]) ? {
            ext: "spx",
            mime: "audio/ogg"
          } : zr(t, [1, 118, 111, 114, 98, 105, 115]) ? {
            ext: "ogg",
            mime: "audio/ogg"
          } : {
            ext: "ogx",
            mime: "application/ogg"
          };
        }

        if (this.check([80, 75]) && (3 === this.buffer[2] || 5 === this.buffer[2] || 7 === this.buffer[2]) && (4 === this.buffer[3] || 6 === this.buffer[3] || 8 === this.buffer[3])) return {
          ext: "zip",
          mime: "application/zip"
        };

        if (this.checkString("ftyp", {
          offset: 4
        }) && 0 != (96 & this.buffer[8])) {
          const e = this.buffer.toString("binary", 8, 12).replace("\0", " ").trim();

          switch (e) {
            case "avif":
            case "avis":
              return {
                ext: "avif",
                mime: "image/avif"
              };

            case "mif1":
              return {
                ext: "heic",
                mime: "image/heif"
              };

            case "msf1":
              return {
                ext: "heic",
                mime: "image/heif-sequence"
              };

            case "heic":
            case "heix":
              return {
                ext: "heic",
                mime: "image/heic"
              };

            case "hevc":
            case "hevx":
              return {
                ext: "heic",
                mime: "image/heic-sequence"
              };

            case "qt":
              return {
                ext: "mov",
                mime: "video/quicktime"
              };

            case "M4V":
            case "M4VH":
            case "M4VP":
              return {
                ext: "m4v",
                mime: "video/x-m4v"
              };

            case "M4P":
              return {
                ext: "m4p",
                mime: "video/mp4"
              };

            case "M4B":
              return {
                ext: "m4b",
                mime: "audio/mp4"
              };

            case "M4A":
              return {
                ext: "m4a",
                mime: "audio/x-m4a"
              };

            case "F4V":
              return {
                ext: "f4v",
                mime: "video/mp4"
              };

            case "F4P":
              return {
                ext: "f4p",
                mime: "video/mp4"
              };

            case "F4A":
              return {
                ext: "f4a",
                mime: "audio/mp4"
              };

            case "F4B":
              return {
                ext: "f4b",
                mime: "audio/mp4"
              };

            case "crx":
              return {
                ext: "cr3",
                mime: "image/x-canon-cr3"
              };

            default:
              return e.startsWith("3g") ? e.startsWith("3g2") ? {
                ext: "3g2",
                mime: "video/3gpp2"
              } : {
                ext: "3gp",
                mime: "video/3gpp"
              } : {
                ext: "mp4",
                mime: "video/mp4"
              };
          }
        }

        if (this.checkString("MThd")) return {
          ext: "mid",
          mime: "audio/midi"
        };
        if (this.checkString("wOFF") && (this.check([0, 1, 0, 0], {
          offset: 4
        }) || this.checkString("OTTO", {
          offset: 4
        }))) return {
          ext: "woff",
          mime: "font/woff"
        };
        if (this.checkString("wOF2") && (this.check([0, 1, 0, 0], {
          offset: 4
        }) || this.checkString("OTTO", {
          offset: 4
        }))) return {
          ext: "woff2",
          mime: "font/woff2"
        };
        if (this.check([212, 195, 178, 161]) || this.check([161, 178, 195, 212])) return {
          ext: "pcap",
          mime: "application/vnd.tcpdump.pcap"
        };
        if (this.checkString("DSD ")) return {
          ext: "dsf",
          mime: "audio/x-dsf"
        };
        if (this.checkString("LZIP")) return {
          ext: "lz",
          mime: "application/x-lzip"
        };
        if (this.checkString("fLaC")) return {
          ext: "flac",
          mime: "audio/flac"
        };
        if (this.check([66, 80, 71, 251])) return {
          ext: "bpg",
          mime: "image/bpg"
        };
        if (this.checkString("wvpk")) return {
          ext: "wv",
          mime: "audio/wavpack"
        };

        if (this.checkString("%PDF")) {
          await e.ignore(1350);
          const t = 10485760,
                r = Buffer.alloc(Math.min(t, e.fileInfo.size));
          return await e.readBuffer(r, {
            mayBeLess: !0
          }), r.includes(Buffer.from("AIPrivateData")) ? {
            ext: "ai",
            mime: "application/postscript"
          } : {
            ext: "pdf",
            mime: "application/pdf"
          };
        }

        if (this.check([0, 97, 115, 109])) return {
          ext: "wasm",
          mime: "application/wasm"
        };

        if (this.check([73, 73])) {
          const e = await this.readTiffHeader(!1);
          if (e) return e;
        }

        if (this.check([77, 77])) {
          const e = await this.readTiffHeader(!0);
          if (e) return e;
        }

        if (this.checkString("MAC ")) return {
          ext: "ape",
          mime: "audio/ape"
        };

        if (this.check([26, 69, 223, 163])) {
          async function t() {
            const t = await e.peekNumber(cr.UINT8);
            let r = 128,
                n = 0;

            for (; 0 == (t & r);) ++n, r >>= 1;

            const i = Buffer.alloc(n + 1);
            return await e.readBuffer(i), i;
          }

          async function r() {
            const e = await t(),
                  r = await t();
            r[0] ^= 128 >> r.length - 1;
            const n = Math.min(6, r.length);
            return {
              id: e.readUIntBE(0, e.length),
              len: r.readUIntBE(r.length - n, n)
            };
          }

          async function n(t, n) {
            for (; n > 0;) {
              const t = await r();

              if (17026 === t.id) {
                return (await e.readToken(new cr.StringType(t.len, "utf-8"))).replace(/\00.*$/g, "");
              }

              await e.ignore(t.len), --n;
            }
          }

          const i = await r();

          switch (await n(0, i.len)) {
            case "webm":
              return {
                ext: "webm",
                mime: "video/webm"
              };

            case "matroska":
              return {
                ext: "mkv",
                mime: "video/x-matroska"
              };

            default:
              return;
          }
        }

        if (this.check([82, 73, 70, 70])) {
          if (this.check([65, 86, 73], {
            offset: 8
          })) return {
            ext: "avi",
            mime: "video/vnd.avi"
          };
          if (this.check([87, 65, 86, 69], {
            offset: 8
          })) return {
            ext: "wav",
            mime: "audio/vnd.wave"
          };
          if (this.check([81, 76, 67, 77], {
            offset: 8
          })) return {
            ext: "qcp",
            mime: "audio/qcelp"
          };
        }

        if (this.checkString("SQLi")) return {
          ext: "sqlite",
          mime: "application/x-sqlite3"
        };
        if (this.check([78, 69, 83, 26])) return {
          ext: "nes",
          mime: "application/x-nintendo-nes-rom"
        };
        if (this.checkString("Cr24")) return {
          ext: "crx",
          mime: "application/x-google-chrome-extension"
        };
        if (this.checkString("MSCF") || this.checkString("ISc(")) return {
          ext: "cab",
          mime: "application/vnd.ms-cab-compressed"
        };
        if (this.check([237, 171, 238, 219])) return {
          ext: "rpm",
          mime: "application/x-rpm"
        };
        if (this.check([197, 208, 211, 198])) return {
          ext: "eps",
          mime: "application/eps"
        };
        if (this.check([40, 181, 47, 253])) return {
          ext: "zst",
          mime: "application/zstd"
        };
        if (this.check([127, 69, 76, 70])) return {
          ext: "elf",
          mime: "application/x-elf"
        };
        if (this.check([79, 84, 84, 79, 0])) return {
          ext: "otf",
          mime: "font/otf"
        };
        if (this.checkString("#!AMR")) return {
          ext: "amr",
          mime: "audio/amr"
        };
        if (this.checkString("{\\rtf")) return {
          ext: "rtf",
          mime: "application/rtf"
        };
        if (this.check([70, 76, 86, 1])) return {
          ext: "flv",
          mime: "video/x-flv"
        };
        if (this.checkString("IMPM")) return {
          ext: "it",
          mime: "audio/x-it"
        };
        if (this.checkString("-lh0-", {
          offset: 2
        }) || this.checkString("-lh1-", {
          offset: 2
        }) || this.checkString("-lh2-", {
          offset: 2
        }) || this.checkString("-lh3-", {
          offset: 2
        }) || this.checkString("-lh4-", {
          offset: 2
        }) || this.checkString("-lh5-", {
          offset: 2
        }) || this.checkString("-lh6-", {
          offset: 2
        }) || this.checkString("-lh7-", {
          offset: 2
        }) || this.checkString("-lzs-", {
          offset: 2
        }) || this.checkString("-lz4-", {
          offset: 2
        }) || this.checkString("-lz5-", {
          offset: 2
        }) || this.checkString("-lhd-", {
          offset: 2
        })) return {
          ext: "lzh",
          mime: "application/x-lzh-compressed"
        };

        if (this.check([0, 0, 1, 186])) {
          if (this.check([33], {
            offset: 4,
            mask: [241]
          })) return {
            ext: "mpg",
            mime: "video/MP1S"
          };
          if (this.check([68], {
            offset: 4,
            mask: [196]
          })) return {
            ext: "mpg",
            mime: "video/MP2P"
          };
        }

        if (this.checkString("ITSF")) return {
          ext: "chm",
          mime: "application/vnd.ms-htmlhelp"
        };
        if (this.check([253, 55, 122, 88, 90, 0])) return {
          ext: "xz",
          mime: "application/x-xz"
        };
        if (this.checkString("<?xml ")) return {
          ext: "xml",
          mime: "application/xml"
        };
        if (this.check([55, 122, 188, 175, 39, 28])) return {
          ext: "7z",
          mime: "application/x-7z-compressed"
        };
        if (this.check([82, 97, 114, 33, 26, 7]) && (0 === this.buffer[6] || 1 === this.buffer[6])) return {
          ext: "rar",
          mime: "application/x-rar-compressed"
        };
        if (this.checkString("solid ")) return {
          ext: "stl",
          mime: "model/stl"
        };
        if (this.checkString("BLENDER")) return {
          ext: "blend",
          mime: "application/x-blender"
        };

        if (this.checkString("!<arch>")) {
          await e.ignore(8);
          return "debian-binary" === (await e.readToken(new cr.StringType(13, "ascii"))) ? {
            ext: "deb",
            mime: "application/x-deb"
          } : {
            ext: "ar",
            mime: "application/x-unix-archive"
          };
        }

        if (this.check([137, 80, 78, 71, 13, 10, 26, 10])) {
          async function t() {
            return {
              length: await e.readToken(cr.INT32_BE),
              type: await e.readToken(new cr.StringType(4, "binary"))
            };
          }

          await e.ignore(8);

          do {
            const r = await t();
            if (r.length < 0) return;

            switch (r.type) {
              case "IDAT":
                return {
                  ext: "png",
                  mime: "image/png"
                };

              case "acTL":
                return {
                  ext: "apng",
                  mime: "image/apng"
                };

              default:
                await e.ignore(r.length + 4);
            }
          } while (e.position + 8 < e.fileInfo.size);

          return {
            ext: "png",
            mime: "image/png"
          };
        }

        if (this.check([65, 82, 82, 79, 87, 49, 0, 0])) return {
          ext: "arrow",
          mime: "application/x-apache-arrow"
        };
        if (this.check([103, 108, 84, 70, 2, 0, 0, 0])) return {
          ext: "glb",
          mime: "model/gltf-binary"
        };
        if (this.check([102, 114, 101, 101], {
          offset: 4
        }) || this.check([109, 100, 97, 116], {
          offset: 4
        }) || this.check([109, 111, 111, 118], {
          offset: 4
        }) || this.check([119, 105, 100, 101], {
          offset: 4
        })) return {
          ext: "mov",
          mime: "video/quicktime"
        };
        if (this.check([239, 187, 191]) && this.checkString("<?xml", {
          offset: 3
        })) return {
          ext: "xml",
          mime: "application/xml"
        };
        if (this.check([73, 73, 82, 79, 8, 0, 0, 0, 24])) return {
          ext: "orf",
          mime: "image/x-olympus-orf"
        };
        if (this.checkString("gimp xcf ")) return {
          ext: "xcf",
          mime: "image/x-xcf"
        };
        if (this.check([73, 73, 85, 0, 24, 0, 0, 0, 136, 231, 116, 216])) return {
          ext: "rw2",
          mime: "image/x-panasonic-rw2"
        };

        if (this.check([48, 38, 178, 117, 142, 102, 207, 17, 166, 217])) {
          async function t() {
            const t = Buffer.alloc(16);
            return await e.readBuffer(t), {
              id: t,
              size: Number(await e.readToken(cr.UINT64_LE))
            };
          }

          for (await e.ignore(30); e.position + 24 < e.fileInfo.size;) {
            const r = await t();
            let n = r.size - 24;

            if (zr(r.id, [145, 7, 220, 183, 183, 169, 207, 17, 142, 230, 0, 192, 12, 32, 83, 101])) {
              const t = Buffer.alloc(16);
              if (n -= await e.readBuffer(t), zr(t, [64, 158, 105, 248, 77, 91, 207, 17, 168, 253, 0, 128, 95, 92, 68, 43])) return {
                ext: "asf",
                mime: "audio/x-ms-asf"
              };
              if (zr(t, [192, 239, 25, 188, 77, 91, 207, 17, 168, 253, 0, 128, 95, 92, 68, 43])) return {
                ext: "asf",
                mime: "video/x-ms-asf"
              };
              break;
            }

            await e.ignore(n);
          }

          return {
            ext: "asf",
            mime: "application/vnd.ms-asf"
          };
        }

        if (this.check([171, 75, 84, 88, 32, 49, 49, 187, 13, 10, 26, 10])) return {
          ext: "ktx",
          mime: "image/ktx"
        };
        if ((this.check([126, 16, 4]) || this.check([126, 24, 4])) && this.check([48, 77, 73, 69], {
          offset: 4
        })) return {
          ext: "mie",
          mime: "application/x-mie"
        };
        if (this.check([39, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], {
          offset: 2
        })) return {
          ext: "shp",
          mime: "application/x-esri-shape"
        };

        if (this.check([0, 0, 0, 12, 106, 80, 32, 32, 13, 10, 135, 10])) {
          await e.ignore(20);

          switch (await e.readToken(new cr.StringType(4, "ascii"))) {
            case "jp2 ":
              return {
                ext: "jp2",
                mime: "image/jp2"
              };

            case "jpx ":
              return {
                ext: "jpx",
                mime: "image/jpx"
              };

            case "jpm ":
              return {
                ext: "jpm",
                mime: "image/jpm"
              };

            case "mjp2":
              return {
                ext: "mj2",
                mime: "image/mj2"
              };

            default:
              return;
          }
        }

        if (this.check([255, 10]) || this.check([0, 0, 0, 12, 74, 88, 76, 32, 13, 10, 135, 10])) return {
          ext: "jxl",
          mime: "image/jxl"
        };
        if (this.check([254, 255, 0, 60, 0, 63, 0, 120, 0, 109, 0, 108]) || this.check([255, 254, 60, 0, 63, 0, 120, 0, 109, 0, 108, 0])) return {
          ext: "xml",
          mime: "application/xml"
        };
        if (this.check([0, 0, 1, 186]) || this.check([0, 0, 1, 179])) return {
          ext: "mpg",
          mime: "video/mpeg"
        };
        if (this.check([0, 1, 0, 0, 0])) return {
          ext: "ttf",
          mime: "font/ttf"
        };
        if (this.check([0, 0, 1, 0])) return {
          ext: "ico",
          mime: "image/x-icon"
        };
        if (this.check([0, 0, 2, 0])) return {
          ext: "cur",
          mime: "image/x-icon"
        };
        if (this.check([208, 207, 17, 224, 161, 177, 26, 225])) return {
          ext: "cfb",
          mime: "application/x-cfb"
        };

        if (await e.peekBuffer(this.buffer, {
          length: Math.min(256, e.fileInfo.size),
          mayBeLess: !0
        }), this.checkString("BEGIN:")) {
          if (this.checkString("VCARD", {
            offset: 6
          })) return {
            ext: "vcf",
            mime: "text/vcard"
          };
          if (this.checkString("VCALENDAR", {
            offset: 6
          })) return {
            ext: "ics",
            mime: "text/calendar"
          };
        }

        if (this.checkString("FUJIFILMCCD-RAW")) return {
          ext: "raf",
          mime: "image/x-fujifilm-raf"
        };
        if (this.checkString("Extended Module:")) return {
          ext: "xm",
          mime: "audio/x-xm"
        };
        if (this.checkString("Creative Voice File")) return {
          ext: "voc",
          mime: "audio/x-voc"
        };

        if (this.check([4, 0, 0, 0]) && this.buffer.length >= 16) {
          const e = this.buffer.readUInt32LE(12);
          if (e > 12 && this.buffer.length >= e + 16) try {
            const t = this.buffer.slice(16, e + 16).toString();
            if (JSON.parse(t).files) return {
              ext: "asar",
              mime: "application/x-asar"
            };
          } catch (e) {}
        }

        if (this.check([6, 14, 43, 52, 2, 5, 1, 1, 13, 1, 2, 1, 1, 2])) return {
          ext: "mxf",
          mime: "application/mxf"
        };
        if (this.checkString("SCRM", {
          offset: 44
        })) return {
          ext: "s3m",
          mime: "audio/x-s3m"
        };
        if (this.check([71]) && this.check([71], {
          offset: 188
        })) return {
          ext: "mts",
          mime: "video/mp2t"
        };
        if (this.check([71], {
          offset: 4
        }) && this.check([71], {
          offset: 196
        })) return {
          ext: "mts",
          mime: "video/mp2t"
        };
        if (this.check([66, 79, 79, 75, 77, 79, 66, 73], {
          offset: 60
        })) return {
          ext: "mobi",
          mime: "application/x-mobipocket-ebook"
        };
        if (this.check([68, 73, 67, 77], {
          offset: 128
        })) return {
          ext: "dcm",
          mime: "application/dicom"
        };
        if (this.check([76, 0, 0, 0, 1, 20, 2, 0, 0, 0, 0, 0, 192, 0, 0, 0, 0, 0, 0, 70])) return {
          ext: "lnk",
          mime: "application/x.ms.shortcut"
        };
        if (this.check([98, 111, 111, 107, 0, 0, 0, 0, 109, 97, 114, 107, 0, 0, 0, 0])) return {
          ext: "alias",
          mime: "application/x.apple.alias"
        };
        if (this.check([76, 80], {
          offset: 34
        }) && (this.check([0, 0, 1], {
          offset: 8
        }) || this.check([1, 0, 2], {
          offset: 8
        }) || this.check([2, 0, 2], {
          offset: 8
        }))) return {
          ext: "eot",
          mime: "application/vnd.ms-fontobject"
        };
        if (this.check([6, 6, 237, 245, 216, 29, 70, 229, 189, 49, 239, 231, 254, 116, 183, 29])) return {
          ext: "indd",
          mime: "application/x-indesign"
        };
        if (await e.peekBuffer(this.buffer, {
          length: Math.min(512, e.fileInfo.size),
          mayBeLess: !0
        }), function (e, t = 0) {
          const r = Number.parseInt(e.toString("utf8", 148, 154).replace(/\0.*$/, "").trim(), 8);
          if (Number.isNaN(r)) return !1;
          let n = 256;

          for (let r = t; r < t + 148; r++) n += e[r];

          for (let r = t + 156; r < t + 512; r++) n += e[r];

          return r === n;
        }(this.buffer)) return {
          ext: "tar",
          mime: "application/x-tar"
        };
        if (this.check([255, 254, 255, 14, 83, 0, 107, 0, 101, 0, 116, 0, 99, 0, 104, 0, 85, 0, 112, 0, 32, 0, 77, 0, 111, 0, 100, 0, 101, 0, 108, 0])) return {
          ext: "skp",
          mime: "application/vnd.sketchup.skp"
        };
        if (this.checkString("-----BEGIN PGP MESSAGE-----")) return {
          ext: "pgp",
          mime: "application/pgp-encrypted"
        };

        if (this.buffer.length >= 2 && this.check([255, 224], {
          offset: 0,
          mask: [255, 224]
        })) {
          if (this.check([16], {
            offset: 1,
            mask: [22]
          })) return this.check([8], {
            offset: 1,
            mask: [8]
          }), {
            ext: "aac",
            mime: "audio/aac"
          };
          if (this.check([2], {
            offset: 1,
            mask: [6]
          })) return {
            ext: "mp3",
            mime: "audio/mpeg"
          };
          if (this.check([4], {
            offset: 1,
            mask: [6]
          })) return {
            ext: "mp2",
            mime: "audio/mpeg"
          };
          if (this.check([6], {
            offset: 1,
            mask: [6]
          })) return {
            ext: "mp1",
            mime: "audio/mpeg"
          };
        }
      }

    }

    e.YC = ot, e.YL = it, e.YR = {
      email: /\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*/,
      phone: /^1[3|4|5|6|7|8][0-9]{9}$/,
      fixedPhone: /(^0\d{2,3}-\d{7,8}$)|(^1[3|4|5|6|7|8][0-9]{9}$)/,
      qq: /[1-9][0-9]{4,}/,
      zipcode: /[1-9]\d{5}(?!\d)/,
      IDCard: /^\d{17}[\d|X|x]|\d{15}$/,
      chinese: /^[\u0391-\uFFE5]+$/,
      chineseNumberLine: /^[\u4E00-\u9FA5A-Za-z0-9_]+$/,
      chineseNumberNoLine: /^[\u4E00-\u9FA5A-Za-z0-9]+$/,
      integerZ: /^[1-9]\d*$/,
      integerF: /^-[1-9]\d*$/,
      number: /^\d+(\.\d+)?$/,
      numberNoMoreThan1: /^\-?([0-9](\d+)?(\.\d{1,1})?$)|(^0$)|(^\d\.\d{1,1})$/,
      numberNoMoreThan2: /^\-?([0-9](\d+)?(\.\d{1,2})?$)|(^0$)|(^\d\.\d{1,2})$/,
      numberNoMoreThan3: /^\-?([0-9](\d+)?(\.\d{1,3})?$)|(^0$)|(^\d\.\d{1,3})$/,
      numberNoMoreThan4: /^\-?([0-9](\d+)?(\.\d{1,4})?$)|(^0$)|(^\d\.\d{1,4})$/,
      greaterZero1: /^(0|[1-9]\d*)(\.\d{1,1})?$/,
      greaterZero2: /^(0|[1-9]\d*)(\.\d{1,2})?$/,
      greaterZero3: /^(0|[1-9]\d*)(\.\d{1,3})?$/,
      greaterZero4: /^(0|[1-9]\d*)(\.\d{1,4})?$/,
      zeroTo100: /^((\d|[1-9]\d)(\.\d{1,2})?$)|(^100|100.0|100.00$)$/,
      zeroTo1000: /^((\d|[1-9]{1,2}\d)(\.\d{1,2})?$)|(^1000|1000.0|1000.00$)$/
    }, e.YS = nt, e.calTimeLen = (e, t, r = "s", n) => {
      let i;

      switch (r) {
        case "m":
          i = 1e3 * wt;
          break;

        case "h":
          i = 1e3 * yt;
          break;

        case "d":
          i = 1e3 * gt;
          break;

        default:
          i = 1e3;
      }

      let o = (ut(t) - ut(e)) / i;
      return o = "number" == typeof n ? Number(o.toFixed(n)) : o, o;
    }, e.checkFileType = (e, t) => {
      const r = new FileReader();
      r.readAsArrayBuffer(e), r.onload = async r => {
        var n, i;

        try {
          const o = new Uint8Array(null === (n = r.target) || void 0 === n ? void 0 : n.result),
                s = await fr.fromBuffer(o),
                a = await Fr(s),
                c = e.name.indexOf(".") > -1 ? (e.name.split(".").pop() || "").toLocaleLowerCase() : "";
          let u = !a || c === a.ext;
          c.indexOf("xls") > -1 && ((null === (i = null == a ? void 0 : a.ext) || void 0 === i ? void 0 : i.indexOf("xls")) > -1 || "cfb" === (null == a ? void 0 : a.ext)) && (u = !0), t(u, a);
        } catch (e) {
          if (!(e instanceof fr.EndOfStreamError)) throw e;
          t(!0, void 0);
        }
      };
    }, e.dateFormat = ft, e.debounce = ar, e.expandMultiple = (e, t, r) => Dt(e)(t, r), e.exportExcel = (e, t = {}) => {
      let r;

      if ("undefined" != typeof window) {
        r = window.localStorage.getItem("token");
        const n = Object.assign(Object.assign({}, t), {
          token: r
        });
        /^https.*/.test(e) || (e = e.replace("http", "https"));
        const i = `${e}?${(e => {
          const t = [];
          return Object.keys(e).forEach(r => {
            We(e[r]) || Z(e[r]) || t.push(`${r}=${e[r]}`);
          }), t.join("&");
        })(n)}`;
        console.log("YWFE下载地址=>", i);
        const o = `alink_${new Date().getTime()}`,
              s = document.createElement("a");
        s.setAttribute("href", i), s.setAttribute("target", "_blank"), s.setAttribute("id", o), document.getElementById(o) || document.body.appendChild(s), s.click(), setTimeout(() => {
          document.body.removeChild(s);
        }, 0);
      }
    }, e.fenToyuan = (e, t) => Rt(100)(e, t), e.fileTypeFromTokenizer = Fr, e.formatData = e => {
      let {
        afterUnit: t
      } = e;
      const {
        type: r,
        value: n,
        isAbs: i,
        beforeUnit: o
      } = e;
      if (-2147483648 === Number(n)) return "无权限";
      if ("text" === r) return n || 0 === n ? n : "-";
      if (isNaN(n) || 0 !== n && !n) return "-";
      let s;

      switch (r) {
        case "yuan":
          s = kt(n, 2);
          break;

        case "fen":
          s = kt(n / 100, 2);
          break;

        case "decimalToPercentSave2Decimal":
          s = kt(100 * n, 2), t = "string" == typeof t ? t : "%";
          break;

        case "decimalToPercentSave0Decimal":
          s = kt(100 * n, 0), t = "string" == typeof t ? t : "%";
          break;

        case "percentSave2Decimal":
          s = kt(n, 2), t = "string" == typeof t ? t : "%";
          break;

        case "percentSave0Decimal":
          s = kt(n, 0), t = "string" == typeof t ? t : "%";
          break;

        case "int":
          s = kt(n, 2, !0, !0);
          break;

        case "decimal":
          s = kt(n, 2, !0, !1);
          break;

        case "secToTimeHM":
          s = pt(n, "{HH}h{mm}m");
          break;

        case "secToTimeChMS":
          s = pt(n, "{mm}分{ss}秒");
          break;

        case "secToTimeMS":
          s = pt(n, "{mm}m{ss}s");
          break;

        case "secToTime":
          s = pt(n, "{DD}d{HH}h{mm}m{ss}s", !1), s = s.replace("00d", "").replace("00h", "");
          break;

        default:
          s = kt(n);
      }

      return s = "-" !== s && i ? s.replace("-", "") : s, "-" === s ? "-" : `${o || ""}${s}${t || ""}`;
    }, e.formatTime = pt, e.gTokg = (e, t) => Rt(1e3)(e, t), e.getBiteLen = e => {
      if ("string" != typeof e) return 0;
      let t = 0;

      for (let r = 0; r < e.length; r++) e.charCodeAt(r) > 127 || 94 === e.charCodeAt(r) ? t += 2 : t += 1;

      return t;
    }, e.getDate = ut, e.getEnv = St, e.getHost = (e, t) => {
      const r = window.configs ? window.configs : {};
      let n = St();
      return {
        basePath: r[e] ? r[e][n].API_BASE_URL + "" + ("mock" === n ? "/" + t : "") : "",
        loginPath: r.AUTH ? r.AUTH[n].APPLICATION_URL : ""
      };
    }, e.getImgUrl = (e, t = 200, r, n) => {
      const i = new RegExp(/resize:(\d+):(\d+)/, "g");
      let o = "";
      return e ? (o = e, o = i.test(o) ? o.replace(i, `resize:${t}:${r || 0}`) : o.indexOf("x-oss-process=image") > -1 || o.indexOf("?") > -1 ? o : `${o}?x-oss-process=image/resize,w_${t},limit_0/quality,q_60`) : o = n || "https://cdn.ywwl.com/bps/ywyun/dianhu/assets/def_goods_img.png", o;
    }, e.getMD5 = e => We(e) || Z(e) ? e : He(String, e) ? or(e) : e, e.getQueries = e => {
      let t = e || window.location.search;
      return t = t.replace("?", ""), Et(t) || {};
    }, e.getQuery = (e, t) => {
      const r = new RegExp(`(\\?|&)${e}=([^&]*)(&|$)`, "i");
      let n = window.location.href;
      const i = r.exec(t || n);
      return null != i && decodeURIComponent(i[2]) || "";
    }, e.getQueryName = (e, t) => {
      let r = t || decodeURI(window.location.href);
      r = r.split("?")[1];
      const n = new Object();

      if (-1 === r.indexOf("?")) {
        const e = r.split("&");

        for (let t = 0; t < e.length; t++) {
          const r = e[t],
                i = r.indexOf("=");
          i > 0 && (n[r.substr(0, i)] = decodeURI(r.substr(i + 1)));
        }
      }

      return n[e] || "";
    }, e.getQueryOfNextPageContext = e => {
      if (!e) return {};
      const {
        pathname: t = "",
        asPath: r = "",
        query: n
      } = e,
            i = ce(Je("/"), Te(e => /\[\w+\]/.test(e)), D(e => Ge("]", "", Ge("[", "", e))))(t),
            o = ce(Je("#"), se(0, 1))(r) || [],
            s = o.length ? ce(Je("/"), Te(e => "" !== e))(o[0]) : [],
            a = i.map((e, t) => ({
        [`${e}`]: s[t]
      }));
      let c = {};
      return i.length ? c = Ye(a) || {} : c.path = s[0] || "desktop", Object.assign(Object.assign({}, n), c);
    }, e.getRoutes = (e, t, r, n) => {
      if (We(e) || We(t)) return {
        menus: [],
        defaultSelectedMenus: [],
        defaultOpenMenus: [],
        defaultMenu: {},
        permissions: []
      };
      let i;
      i = "/" !== n && "/undefined" !== n && "/null" !== n ? rt("/", n) : [];
      let o = [],
          s = [],
          a = {};

      const c = e => {
        const n = t[e.url];
        if (!n) return {};
        const i = 0 === e.url.indexOf("/") ? e.url || "" : `/${e.url}`,
              c = {
          path: i,
          param: n.param || "",
          url: n.url,
          name: e.resourceId,
          meta: {
            title: e.name,
            icon: n.icon
          },
          component: n.component,
          hidden: n.hidden,
          origion_url: i
        };
        return ((null == r ? void 0 : r.primaryDefaultMenu) && e.url === (null == r ? void 0 : r.primaryDefaultMenu) || (null == r ? void 0 : r.secondaryDefaultMenu) && e.url === (null == r ? void 0 : r.secondaryDefaultMenu)) && (o = [e.resourceParentId], s = [e.resourceId], a = c), c;
      },
            u = [];

      let f = [];

      if (e.map(e => {
        const r = [],
              n = t[e.url],
              i = xt({
          key: "resourceType",
          value: "BUTTON"
        }, "childMenus", e);

        if (i && i.length) {
          const e = ce(D(W("url")))(i);
          f = _e(f, e);
        }

        if (e.childMenus && e.childMenus.length) {
          xt({
            key: "resourceId"
          }, "childMenus", e.childMenus).map(e => {
            if (e) {
              const t = c(e);
              t && t.name && r.push(t);
            }
          });
        }

        const o = c(e);
        n && u.push(Object.assign(Object.assign({}, o), {
          children: r
        }));
      }), i.length && u.length) {
        const e = bt(i, u);
        s = e.defaultSelectedMenus, o = e.defaultOpenMenus, a = e.defaultMenu;
      } else if (!s.length || !o.length || !a) {
        const e = vt(u);
        s = e.defaultSelectedMenus, o = e.defaultOpenMenus, a = e.defaultMenu;
      }

      return {
        menus: u,
        defaultSelectedMenus: s,
        defaultOpenMenus: o,
        defaultMenu: a,
        permissions: f
      };
    }, e.humpToLine = function (e) {
      return "string" != typeof e ? e : e.replace(/([A-Z])/g, "_$1").toLowerCase();
    }, e.isChinaMobile = e => /^((\+86)|(86)|(086))?(13[0-9]|15[0-9]|166|17[0-9]|18[0-9]|14[0-9]|19[0-9])[0-9]{8}$/.test(String(e)), e.isDate = ct, e.isURL = e => /^((?:(?:https?|ftp):|)\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,}))\.?)(?::\d{2,5})?(?:[/?#]\S*)?$/i.test(String(e)), e.joinPath = e => {
      if (!e || We(e)) return "";
      const t = Object.keys(e);
      let r = "";
      return t.map(t => {
        e[t] && "search" !== t && (r += `/${e[t]}`), "search" === t && (r += e[t]);
      }), r;
    }, e.kgTog = (e, t) => Dt(1e3)(e, t), e.lineToHump = function (e) {
      return "string" != typeof e ? e : e.replace(/\_(\w)/g, function (e, t) {
        return t.toUpperCase();
      });
    }, e.loadScript = function (e, t, r) {
      return new Promise((n, i) => {
        const o = document.createElement("script");
        var s, a;
        o.id = t, o.readyState ? (a = () => n(o), (s = o).onreadystatechange = function () {
          "loaded" !== s.readyState && "complete" !== s.readyState || (s.onreadystatechange = null, a());
        }) : function (e, t, r) {
          st(e, "load", t), st(e, "error", r);
        }(o, () => n(o), () => {
          i(), document.removeChild(document.getElementById(t));
        }), r && at.forEach(e => {
          if (e in r) {
            const t = r[e];
            "string" == typeof t ? o.setAttribute(e, t) : "boolean" == typeof t && (o[e] = t);
          }
        }), o.src = e.js, document.body.appendChild(o);
      });
    }, e.maxSubstrWord = (e, t = 5, r = !0) => "string" == typeof e ? r ? `${e.substr(0, t)}...` : e.substr(0, t) : e, e.niceNumShow = (e, t = !1) => {
      if (0 !== e && !e || "-" === e) return [{
        type: "num",
        text: "-"
      }];
      const r = [],
            n = e.toString().split("");
      let i,
          o = "";
      return n.forEach(e => {
        let n = /^[0-9\\.]*$/.test(e) ? "num" : "character";
        n = "." === e && t || "num" === n && "suffixNum" === i ? "suffixNum" : n, i !== n && i ? (r.push({
          type: i,
          text: o
        }), o = e) : o = `${o}${e}`, i = n;
      }), i = i || "num", r.push({
        type: i,
        text: o
      }), r;
    }, e.numberToThousand = (e, t = ",") => We(e) || Z(e) ? e : isNaN(Number(e)) ? NaN : e.toString().replace(/\d+/, e => e.replace(/(\d)(?=(\d{3})+$)/g, e => `${e}${t}`)), e.omadownload = (e, t, r) => {
      const n = new Uint8Array(e);

      try {
        const e = String.fromCharCode.apply(null, n);
        if (e.indexOf("success") > -1) return JSON.parse(e);
      } catch (e) {
        console.warn(e);
      }

      const i = window.URL.createObjectURL(new Blob([e], {
        type: r || "application/vnd.ms-excel"
      })),
            o = document.createElement("a");
      o.style.display = "none", o.href = i, o.setAttribute("download", decodeURIComponent(t)), document.body.appendChild(o), o.click(), o.remove();
    }, e.parseParams = e => Q(e).join("/"), e.pickAll = (e, t, r) => {
      const n = {
        reduce: ee((r, i, o) => {
          const s = r(i, o);

          if (!(e => Pe([t], e))(o)) {
            return s && s.length ? D(t => t[e] || t)(s) : [];
          }

          return o[t] ? o[t].reduce(n.reduce(r), s) : s;
        }),
        find: ee((e, t, r) => ce(D(n.reduce(e, t)), Le)(r))
      },
            i = (r, n) => n[e] ? r.concat([Qe([t], n)]) : r;

      switch (re(r)) {
        case "Object":
          return n.reduce(i, [])(r);

        case "Array":
          return n.find(i, [])(r);

        default:
          return r;
      }
    }, e.querystring = Tt, e.reduceMultiple = (e, t, r) => Rt(e)(t, r), e.reduceParams = e => {
      if (!e) return {};
      if (e instanceof FormData) return e;
      return Te(e => !Z(e) && !We(e))(e) || {};
    }, e.remove = function (e, t) {
      const r = [];
      if (!e || !e.length) return r;
      let n = -1,
          i = [],
          o = e.length;

      for (; ++n < o;) {
        const o = e[n];
        t(o, n, e) && (r.push(o), i.push(n));
      }

      return function (e, t) {
        let r = e ? t.length : 0,
            n = r - 1,
            i = 0;

        for (; r--;) {
          const o = t[r];
          r != n && o === i || (i = o, Array.prototype.splice.call(e, o, 1));
        }
      }(e, i), r;
    }, e.round = (e, t = 0) => Math.round(Math.random() * (e - t) + t), e.slicePath = (e, t = 0, r) => {
      const n = rt("/", e);
      r || (r = n.length - 1);
      return "/" + n.slice(t, r).join("/");
    }, e.split = rt, e.strPad = lt, e.sum = (e, t) => {
      let r = new $t(0);

      const n = (e, t) => {
        "array" === Ct(e) && Array.isArray(e) && e.forEach(e => {
          isNaN(e) || (r = r.plus(Number(e))), "object" === Ct(e) && void 0 !== t && null !== e[t] && (r = r.plus(Number(e[t]))), "array" === Ct(e) && Array.isArray(e) && n(e, t);
        });
      };

      return n(e, t), Number(r);
    }, e.throttle = function (e, t, r) {
      let n = !0,
          i = !0;
      if ("function" != typeof e) throw new TypeError("Expected a function");
      return sr(r) && (n = "leading" in r ? !!r.leading : n, i = "trailing" in r ? !!r.trailing : i), ar(e, t, {
        leading: n,
        trailing: i,
        maxWait: t
      });
    }, e.timeFlow = (e, t, r = "HH:mm:ss") => ft(ut(e) + 1e3 * t, r), e.toDate = e => {
      if (ct(e)) {
        let t = e;
        return e instanceof Date ? t = e : "string" == typeof e && String(e).length >= 13 && !isNaN(e) ? t = Number(e) : "string" == typeof e && e.includes("-") && (t = e.replace(/-/g, "/")), new Date(t).getTime();
      }

      return 0;
    }, e.transformShortNum = kt, e.translateMapToArray = (e, t = !1, r = "全部", n = null) => {
      let i = t ? [{
        label: r,
        value: n
      }] : [];

      for (const [t, r] of e.entries()) i.push({
        label: r,
        value: t
      });

      return i;
    }, e.treeFind = (e, t, r) => {
      const n = {
        reduce: ee((r, i, s) => {
          const a = r(i, s);
          return ((e => t && Pe([t], e) && !We(e[t]))(s) || s[e.key] !== e.value) && t && s[t] && !We(s[t]) ? n.reduce(o, a)(s[t]) : a;
        }),
        find: ee((e, t, r) => ce(D(n.reduce(e, t)), Le, et(t))(r))
      },
            i = (r, n) => e.value && n[e.key] && n[e.key] === e.value || !e.value && n[e.key] ? t ? r.concat([Qe([t], n)]) : r.concat([n]) : r,
            o = (e, t) => {
        const r = Object.keys(t),
              o = [];
        return r.map(e => {
          o.push(t[e]);
        }), n.find(i, e)(o);
      };

      switch (re(r)) {
        case "Object":
          return n.reduce(o, [])(r);

        case "Array":
          return n.find(i, [])(r);

        default:
          return [];
      }
    }, e.treePaths = (e, t, r) => {
      const n = {
        reduce: ee((r, i, s, a) => {
          const c = r(i, s, a);
          return (e => Pe([t], e) && !We(e[t]))(a) || a[e.key] !== e.value ? a[t] && !We(a[t]) ? n.reduce(o, [], Qe([t], a))(a[t]) : c : (s.path && c.splice(0, 0, s.path), c);
        }),
        find: ee((e, t, r, i) => ce(D(n.reduce(e, t, r)), Le)(i))
      },
            i = (t, r, n) => e.value && n[e.key] && n[e.key] === e.value || !e.value && n[e.key] ? t.concat([n.path]) : t,
            o = (e, t, r) => {
        if (!r || We(r)) return e;
        const o = Object.keys(r),
              s = [];
        return o.map(e => {
          s.push(r[e]);
        }), n.find(i, e, t)(s);
      };

      switch (re(r)) {
        case "Object":
          return n.reduce(o, [], {})(r);

        case "Array":
          return n.find(i, [], {})(r);

        default:
          return [];
      }
    }, e.treeReduce = xt, e.trimAll = tt, e.trimParams = qt, e.unLoadScript = function (e) {
      if (document.getElementById(e)) {
        const t = document.getElementById(e);
        document.body.removeChild(t);
      }
    }, e.uuidv4 = () => Jt(), e.wanToyuan = (e, t) => Dt(1e4)(e, t), e.yuanTofen = (e, t) => Dt(100)(e, t), e.yuanTowan = (e, t) => Rt(1e4)(e, t), Object.defineProperty(e, "__esModule", {
      value: !0
    });
  });
})(ywfeUtils_umd, ywfeUtils_umd.exports);

const USER_UNLOGIN = 17020001;
const USER_TOKEN_INVALID = 17020002;
const USER_TOKEN_NULL = 17020003;
const TOKEN_FAIL_OLD_1 = -2;
const TOKEN_FAIL_OLD_2 = 1000;
const BIZ_MODULE_NOTFOUND = 17031000;
const GO_TO_LOGIN = [
    USER_UNLOGIN,
    USER_TOKEN_INVALID,
    USER_TOKEN_NULL,
    TOKEN_FAIL_OLD_1,
    TOKEN_FAIL_OLD_2,
];
const NO_MESSAGE = [BIZ_MODULE_NOTFOUND];

const instance = axios.create({
    timeout: 30 * 1000,
});
instance.interceptors.request.use(config => {
    var _a, _b, _c;
    const configData = config;
    let actionId = '';
    let pathUrlStart = ((_a = config.url) === null || _a === void 0 ? void 0 : _a.indexOf('ywwl.com')) || 0;
    let pathUrlEnd = ((_b = config.url) === null || _b === void 0 ? void 0 : _b.indexOf('?')) || 0;
    pathUrlEnd = pathUrlEnd === -1 ? 0 : pathUrlEnd;
    let md5Url = ywfeUtils_umd.exports.getMD5((_c = config.url) === null || _c === void 0 ? void 0 : _c.slice(pathUrlStart + 8, pathUrlEnd)) || '';
    let aPageUrl = window.location.pathname.split('/');
    let pageUrl = aPageUrl[aPageUrl.length - 1];
    const pageActions = window.actionRelData ? window.actionRelData[md5Url] : void (0);
    if (typeof pageActions !== 'undefined') {
        actionId = pageActions[pageUrl] || '';
    }
    if (typeof window !== 'undefined') {
        const token = localStorage.getItem('token') || '';
        const isGray = localStorage.getItem('isGray') === 'true' || false;
        const grayParams = localStorage.getItem('grayParams') || '';
        configData.headers = Object.assign(Object.assign({}, configData.headers), { 'X-TOKEN': token, 'x-gray': isGray, 'grayParams': grayParams, 'actionId': actionId, 'fingerprint': window.localStorage.getItem('fp'), 'extendParams': JSON.stringify({}) });
    }
    return configData;
}, error => {
    return Promise.reject(error);
});
instance.interceptors.response.use((res) => {
    const { data, config } = res;
    const { success, code } = data;
    const isFormData = typeof data === 'string';
    const isArraybuffer = res.config.responseType === 'arraybuffer';
    if (isFormData || isArraybuffer) {
        if (data) {
            return data;
        }
        else {
            if (typeof window !== 'undefined') {
                antd.message.error(isFormData ? '图片上传失败' : '下载失败');
            }
            return Promise.reject();
        }
    }
    if (success) {
        return data.data || success;
    }
    else {
        if (GO_TO_LOGIN.indexOf(Number(code)) >= 0 &&
            typeof window !== 'undefined') {
            const projectId = window.localStorage.getItem('projectId');
            const { loginHost = '' } = config;
            let pathname = window.location.pathname;
            let search = window.location.search;
            search = search.replace('?', '');
            let query = ywfeUtils_umd.exports.querystring.parse(search);
            query = Object.assign(Object.assign({}, query), { projectId: projectId || query.projectId });
            window.top.location.href = `${loginHost}/login?${ywfeUtils_umd.exports.querystring.stringify(query)}&pathname=${encodeURIComponent(pathname)}&targetUrl=${encodeURIComponent(window.location.pathname + window.location.search)}`;
        }
        if (typeof window !== 'undefined' && NO_MESSAGE.indexOf(code) < 0) {
            antd.message.error(data.msg || data.message);
        }
        return Promise.reject(data);
    }
}, (error) => {
    if (typeof window !== 'undefined') {
        antd.message.error('网络繁忙, 休息一会儿再来吧~ ^_^！');
    }
    return Promise.reject(error);
});
class Request {
    static get(url, config) {
        return instance.get(url, config);
    }
    static post(url, data, config) {
        return instance.post(url, data, config);
    }
}

module.exports = Request;
//# sourceMappingURL=ywfe-request.cjs.js.map
